/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.jms.listener;

import java.lang.reflect.Method;
import javax.jms.ExceptionListener;
import midstream.framework.logging.EventLogger;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.support.StaticMethodMatcherPointcutAdvisor;
import org.springframework.beans.factory.BeanNameAware;

/**
 * Provides JMS receiver wrapper that automatically performs event and
 * performance logging around the <code>onMessage</code> and
 * <code>onException</code> methods of the target message/exception listener.
 *
 * @author Juan Hall
 */
public class DefaultMessageListenerContainer
    extends org.springframework.jms.listener.DefaultMessageListenerContainer
    implements BeanNameAware {

  private String beanName;

  public void setBeanName(String name) {
    this.beanName = name;
  }

  public void afterPropertiesSet() {
    super.afterPropertiesSet();

    MethodInterceptor interceptor = new MethodInterceptor() {
      public Object invoke(MethodInvocation methodInvocation) throws Throwable {
        EventLogger log = new EventLogger(
            beanName + "." + methodInvocation.getMethod().getName());

        try {
          log.logBegin();
          return methodInvocation.proceed();
        } catch (Throwable th) {
          log.logError(th);
          throw th;
        } finally {
          log.logEnd();
        }
      }
    };

    ProxyFactory factory = new ProxyFactory(this.getMessageListener());
    factory.addAdvisor(new StaticMethodMatcherPointcutAdvisor(interceptor) {
      public boolean matches(Method method, Class aClass) {
        return method.getName().startsWith("onMessage");
      }
    });

    this.setMessageListener(factory.getProxy());

    if (this.getExceptionListener() != null) {
      factory = new ProxyFactory(this.getExceptionListener());
      factory.addAdvisor(new StaticMethodMatcherPointcutAdvisor(interceptor) {
        public boolean matches(Method method, Class aClass) {
          return method.getName().startsWith("onException");
        }
      });

      this.setExceptionListener((ExceptionListener) factory.getProxy());
    }
  }
}
