/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.logging;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;
import org.apache.log4j.Logger;

/**
 * Performs logging of event records and performance tracking. The logger
 * outputs records to the <code>event-logger</code> appender in the following
 * formats: <ul> <li>Begin Event: BEG|&lt;Thread ID&gt;-&lt;Current Time in
 * Millis&gt;|&lt;Event ID&gt;</li> <li>Error Event: ERR|&lt;Thread
 * ID&gt;-&lt;Current Time in Millis&gt;|&lt;Event ID&gt;|&lt;Exception
 * Message&gt;</li> <li>End Event: END|&lt;Thread ID&gt;-&lt;Current Time in
 * Millis&gt;|&lt;Event ID&gt;</li> </ul> The performance metrics can be viewed
 * from the <code>Monitoring View</code> via the <code>MidStream</code> plugin
 * or via the application's <code>MonitorSummary</code> URL.
 *
 * @author Juan Hall
 * @see midstream.framework.monitoring.MonitoringController
 */
public class EventLogger {

  private static final Logger log = Logger.getLogger("event-logger");

  long startTime;

  private String eventId;

  Monitor monitor;

  /**
   * Initializes the event ID for the records to be logged.
   *
   * @param eventId a unique identifier for the type of record being logged
   */
  public EventLogger(String eventId) {
    this.eventId = eventId;
  }

  /**
   * Logs a <em>begin</em> entry record in the format of:
   * <pre>BEG|&lt;Thread ID&gt;-&lt;Current Time in Millis&gt;|&lt;Event
   * ID&gt;</pre>
   */
  public void logBegin() {
    if (monitor == null) {
      startTime = getStartTime();
      monitor = MonitorFactory.start(this.eventId);

      log.info(String.format("BEG|%d-%d|%s", getThreadId(),
          startTime, this.eventId));
    }
  }

  long getThreadId() {
    return Thread.currentThread().getId();
  }

  long getStartTime() {
    return System.currentTimeMillis();
  }

  /**
   * Logs an <em>error</em> entry record in the format of:
   * <pre>ERR|&lt;Thread ID&gt;-&lt;Current Time in Millis&gt;|&lt;Event
   * ID&gt;|&lt;Exception Message&gt;</pre>
   */
  public void logError(Throwable th) {
    if (monitor != null) {
      log.info(String.format("ERR|%d-%d|%s|%s", getThreadId(),
          startTime, this.eventId, th.getMessage()), th);
    }
  }

  /**
   * Logs a <em>end</em> entry record in the format of:
   * <pre>END|&lt;Thread ID&gt;-&lt;Current Time in Millis&gt;|&lt;Event
   * ID&gt;</pre>
   */
  public void logEnd() {
    if (monitor != null) {
      log.info(String.format("END|%d-%d|%s", getThreadId(),
          startTime, this.eventId));

      monitor.stop();
    }
  }
}
