/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import java.lang.reflect.Method;
import midstream.framework.logging.EventLogger;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.axis.client.Stub;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.framework.ProxyFactoryBean;
import org.springframework.aop.support.StaticMethodMatcherPointcutAdvisor;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.InitializingBean;

/**
 * Provides a factory bean for creating Web Service client beans based on the <a
 * href="http://ws.apache.org/axis/">Apache Axis</a> API. The factory creates
 * proxies that automatically perform event and performance logging in addition
 * to the setting of the underlying socket's read timeout. The default timeout
 * value is 10 seconds.
 *
 * @author Juan Hall
 */
public class AxisClientProxyFactoryBean extends ProxyFactoryBean implements
    InitializingBean, BeanNameAware {

  public void afterPropertiesSet() throws Exception {
    this.setProxyTargetClass(true);
    this.addAdvisor(new StaticMethodMatcherPointcutAdvisor(
        new ServiceLocatorMethodInterceptor(beanName, timeout)) {

      private static final long serialVersionUID = 1L;

      public boolean matches(Method method, Class arg1) {
        return method.getName().matches("get.*Port");
      }
    });
  }

  private int timeout = 10;

  /**
   * Assigns the value for the client's read timeout. The value should be
   * specified in seconds.
   *
   * @param timeout how long (in seconds) the client should wait to read
   *                responses from the service
   */
  public void setTimeout(int timeout) {
    this.timeout = timeout;
  }

  private String beanName;

  public void setBeanName(String beanName) {
    this.beanName = beanName;
  }
}

/**
 * Provides a method interceptor around the <code>Axis</code> generated service
 * locator for the Web Service in order to set the read timeout and proxy the
 * underlying <code>port</code> class to perform event/performance logging.
 *
 * @author Juan Hall
 */
class ServiceLocatorMethodInterceptor implements MethodInterceptor {

  private String beanName;

  private int timeout;

  public ServiceLocatorMethodInterceptor(String beanName, int timeout) {
    this.beanName = beanName;
    this.timeout = timeout;
  }

  public Object invoke(MethodInvocation inv) throws Throwable {
    final Object port = inv.proceed();

    ((Stub) port).setTimeout(timeout * 1000);
    ProxyFactory proxyFactory = new ProxyFactory(port);
    proxyFactory.setProxyTargetClass(true);
    proxyFactory.addAdvisor(new StaticMethodMatcherPointcutAdvisor(
        new PortMethodInterceptor(this.beanName)) {
      public boolean matches(Method method, Class arg1) {
        return true;
      }
    });

    return proxyFactory.getProxy();
  }
}

/**
 * Provides a method interceptor around the <code>Axis</code> generated
 * <code>port</code> class of the Web Service in order to perform
 * event/performance logging.
 *
 * @author Juan Hall
 */
class PortMethodInterceptor implements MethodInterceptor {

  private String beanName;

  public PortMethodInterceptor(String beanName) {
    this.beanName = beanName;
  }

  public Object invoke(MethodInvocation invocation) throws Throwable {
    EventLogger log =
        new EventLogger(this.beanName + "." + invocation.getMethod().getName());

    try {
      log.logBegin();
      return invocation.proceed();
    } catch (Throwable th) {
      log.logError(th);
      throw th;
    } finally {
      log.logEnd();
    }
  }
}