/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import java.util.Properties;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.factory.config.PropertyOverrideConfigurer;
import org.springframework.context.support.AbstractXmlApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * Provides a client side helper class for obtaining handles to remoted
 * (Java/HTTP) service endpoints for distributed service invocations.
 *
 * @author Juan Hall
 */
public final class ServiceLocator {

  /**
   * Exception type thrown when errors occur processing a service request.
   */
  public static class ServiceException extends Exception {
    public ServiceException(String string, Throwable throwable) {
      super(string, throwable);
    }
  }

  static AbstractXmlApplicationContext context =
      new ClassPathXmlApplicationContext(
          "classpath*:/META-INF/**/clientContext.xml");

  /**
   * Returns a proxy for the requested service (if it exists).
   *
   * @param serviceName the name of the service (as it appears in the client
   *                    context files) to retrieve
   * @return a proxy to the service endpoint
   * @throws ServiceException if an error occurs
   */
  public synchronized static Object getService(final String serviceName)
      throws ServiceException {
    try {
      return context.getBean(serviceName);
    } catch (Exception e) {
      throw new ServiceException("Unable to retrieve service: " + serviceName,
          e);
    }
  }

  /**
   * Returns the URL that client is currently configured to communicate with the
   * specified service.
   *
   * @param serviceName the name of the service of interest
   * @return the URL configured to communicate with the desired service
   * @throws ServiceException if an error occurs
   */
  public synchronized static String getServiceUrl(final String serviceName)
      throws ServiceException {
    try {
      MutablePropertyValues propertyValues =
          context.getBeanFactory().getBeanDefinition(serviceName)
              .getPropertyValues();
      return
          (String) propertyValues.getPropertyValue("serviceUrl").getValue();
    } catch (Exception e) {
      throw new ServiceException(
          "Unable to retrieve the service URL for: " + serviceName, e);
    }
  }

  /**
   * Assigns a new URL to communicate with the specified service.
   *
   * @param serviceName the name of the desired service
   * @param url         the new URL to communicate with the service
   * @throws ServiceException if an error occurs
   */
  public synchronized static void setServiceUrl(final String serviceName,
                                                final String url)
      throws ServiceException {
    try {
      String currentUrl = getServiceUrl(serviceName);

      if (!currentUrl.equals(url)) {
        PropertyOverrideConfigurer overrideConfigurer =
            new PropertyOverrideConfigurer();
        overrideConfigurer.setBeanNameSeparator(">");

        Properties props = new Properties();
        props.put(serviceName + ">serviceUrl", url);

        overrideConfigurer.setProperties(props);

        context.addBeanFactoryPostProcessor(overrideConfigurer);
        context.refresh();
      }
    } catch (Exception e) {
      throw new ServiceException(
          "Unable to set the service URL for: " + serviceName, e);
    }
  }
}