/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import java.io.BufferedInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.security.KeyStore;
import javax.net.SocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import midstream.framework.logging.EventLogger;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.util.FileCopyUtils;

/**
 * Provides a POJO for sending requests to a service via a simple socket.
 *
 * @author Juan Hall
 */
public class SocketClientImpl implements SocketClient, BeanNameAware {

  private static final Logger log = Logger.getLogger(SocketClientImpl.class);

  private String host;

  private int port;

  private int connectionTimeout = 10;

  private int readTimeout = 10;

  private boolean secure;

  private String secureProtocol;

  private String keyAlgorithm;

  private String keyStoreType;

  private String keyStorePassphrase;

  private String keyStoreLocation;

  private String beanName;

  /**
   * Utilizes a <code>SocketClientDelegate</code> callback to for handling
   * communication with the underlying socket.
   *
   * @param delegate the callback to use for reading/writing to the socket
   * @throws IOException if an error occurs
   */
  public void execute(SocketClientDelegate delegate) throws IOException {
    EventLogger log =
        new EventLogger(this.beanName + "." + "execute[delegate]");

    try {
      log.logBegin();
      doExecute(delegate, null);
    } catch (IOException ex) {
      log.logError(ex);
      throw ex;
    } finally {
      log.logEnd();
    }
  }

  /**
   * Writes the given bytes to the socket and returns the response as an array
   * of bytes.
   *
   * @param bytes the request represented as an array of bytes
   * @return the response formatted as an array of bytes
   * @throws IOException if an error occurs
   */
  public byte[] execute(final byte[] bytes) throws IOException {
    EventLogger log = new EventLogger(this.beanName + "." + "execute[byte]");

    try {
      log.logBegin();
      return doExecute(null, bytes);
    } catch (IOException ex) {
      log.logError(ex);
      throw ex;
    } finally {
      log.logEnd();
    }
  }

  byte[] doExecute(SocketClientDelegate delegate, final byte[] bytes)
      throws IOException {
    Socket socket = null;
    BufferedInputStream in = null;
    DataOutputStream out = null;

    try {
      if (this.secure) {
        try {
          SSLContext ctx = SSLContext.getInstance(this.secureProtocol);
          KeyManagerFactory kmf = KeyManagerFactory
              .getInstance(this.keyAlgorithm);
          KeyStore ks = KeyStore.getInstance(this.keyStoreType);
          char[] passphrase = this.keyStorePassphrase == null ? null
              : this.keyStorePassphrase.toCharArray();

          ks.load(new FileInputStream(this.keyStoreLocation), passphrase);
          kmf.init(ks, passphrase);

          TrustManagerFactory tmf = TrustManagerFactory
              .getInstance(this.keyAlgorithm);
          tmf.init(ks);

          ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

          socket = createSecureSocket(ctx);
        } catch (Exception e) {
          final IOException ioe = new IOException(
              "Unable to create secure socket");
          ioe.initCause(e);
          throw ioe;
        }
      } else {
        socket = createSocket();
      }

      socket.setSoTimeout(this.readTimeout * 1000);
      socket.connect(new InetSocketAddress(this.host, this.port),
          this.connectionTimeout * 1000);

      in = new BufferedInputStream(socket.getInputStream());
      out = new DataOutputStream(socket.getOutputStream());

      if (delegate != null) {
        delegate.writeRequest(out);
      } else {
        out.write(bytes);
      }
      out.flush();

      if (delegate != null) {
        delegate.readResponse(in);
        return null;
      } else {
        return FileCopyUtils.copyToByteArray(in);
      }
    } finally {
      try {
        if (out != null) {
          out.close();
        }
      } catch (IOException e) {
        log.warn("Error closing socket output stream", e);
      }

      try {
        if (in != null) {
          in.close();
        }
      } catch (IOException e) {
        log.warn("Error closing socket input stream", e);
      }

      try {
        if (socket != null) {
          socket.close();
        }
      } catch (IOException e) {
        log.warn("Error closing socket", e);
      }
    }
  }

  protected Socket createSocket() throws IOException {
    return SocketFactory.getDefault().createSocket();
  }

  protected Socket createSecureSocket(SSLContext ctx) throws IOException {
    return ctx.getSocketFactory().createSocket();
  }

  public void setHost(String host) {
    this.host = host;
  }

  public void setPort(int port) {
    this.port = port;
  }

  public void setConnectionTimeout(int connectionTimeout) {
    this.connectionTimeout = connectionTimeout;
  }

  public void setReadTimeout(int timeout) {
    this.readTimeout = timeout;
  }

  /**
   * Indicates whether the socket should utilizes SSL for its communication.
   *
   * @param secure <code>true</code> if SSL should be used, otherwise
   *               <code>false</code>
   */
  public void setSecure(boolean secure) {
    this.secure = secure;
  }

  public void setSecureProtocol(String secureProtocol) {
    this.secureProtocol = secureProtocol;
  }

  public void setKeyAlgorithm(String keyAlgorithm) {
    this.keyAlgorithm = keyAlgorithm;
  }

  public void setKeyStoreType(String keyStoreType) {
    this.keyStoreType = keyStoreType;
  }

  public void setKeyStorePassphrase(String keyStorePassphrase) {
    this.keyStorePassphrase = keyStorePassphrase;
  }

  public void setKeyStoreLocation(String keyStoreLocation) {
    this.keyStoreLocation = keyStoreLocation;
  }

  public void setBeanName(String beanName) {
    this.beanName = beanName;
  }
}