/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.xmlinvoker;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import midstream.framework.logging.EventLogger;
import org.jibx.runtime.BindingDirectory;
import org.jibx.runtime.IBindingFactory;
import org.jibx.runtime.IMarshallingContext;
import org.jibx.runtime.IUnmarshallingContext;
import org.jibx.runtime.JiBXException;
import org.springframework.remoting.httpinvoker.HttpInvokerServiceExporter;
import org.springframework.remoting.support.RemoteInvocation;
import org.springframework.remoting.support.RemoteInvocationResult;

/**
 * @author Juan Hall
 */
public class XmlHttpInvokerServiceExporter extends HttpInvokerServiceExporter {

  private String serviceMethod;

  private Class requestClass;

  private Class responseClass;

  @Override
  public void afterPropertiesSet() {
    if (serviceMethod == null || "".equals(serviceMethod.trim())) {
      throw new IllegalArgumentException(
          "A valid service method must be specified");
    }

    if (requestClass == null) {
      throw new IllegalArgumentException(
          "A valid request class must be specified");
    }

    if (responseClass == null) {
      throw new IllegalArgumentException(
          "A valid response class must be specified");
    }

    super.afterPropertiesSet();
  }

  @Override
  protected RemoteInvocation readRemoteInvocation(HttpServletRequest request)
      throws IOException, ClassNotFoundException {
    IBindingFactory bfact;
    Object obj;

    try {
      bfact = getBindingFactory(this.requestClass);
      IUnmarshallingContext uctx = bfact.createUnmarshallingContext();
      obj = uctx.unmarshalDocument(request.getInputStream(), null);
    } catch (Throwable e) {
      return new InvalidRemoteInvocation(this.serviceMethod, e);
    }

    return new RemoteInvocation(this.serviceMethod,
        new Class[]{this.requestClass}, new Object[]{obj});
  }

  IBindingFactory getBindingFactory(Class clazz) throws JiBXException {
    return BindingDirectory.getFactory(clazz);
  }

  private static final String EXCEPTION_TEMPLATE =
      "<?xml version=\"1.0\" encoding=\"UTF-8\"?><exception><type>%s</type><message>%s</message><stacktrace>%s</stacktrace></exception>";

  @Override
  protected void writeRemoteInvocationResult(HttpServletRequest request,
                                             HttpServletResponse response,
                                             RemoteInvocationResult invocationResult)
      throws IOException {
    IBindingFactory bfact;

    try {
      if (invocationResult.hasException()) {
        outputExceptionResponse(response, invocationResult.getException());
      } else {
        bfact = getBindingFactory(this.responseClass);
        IMarshallingContext mctx = bfact.createMarshallingContext();
        mctx.marshalDocument(invocationResult.getValue(), "UTF-8", null,
            response.getOutputStream());
      }
    } catch (Throwable e) {
      outputExceptionResponse(response, e);
    }
  }

  void outputExceptionResponse(HttpServletResponse response, Throwable e)
      throws IOException {
    StringWriter stackTrace = new StringWriter();
    e.printStackTrace(new PrintWriter(stackTrace));

    String result = String
        .format(EXCEPTION_TEMPLATE, e.getClass().getName(), e.getMessage(),
            stackTrace.getBuffer().toString());
    response.getOutputStream().print(result);
  }

  @Override
  protected RemoteInvocationResult invokeAndCreateResult(
      RemoteInvocation invocation, Object targetObject) {
    EventLogger log =
        new EventLogger(this.getServiceInterface().getSimpleName() + "." +
            invocation.getMethodName());

    try {
      log.logBegin();

      if (invocation instanceof InvalidRemoteInvocation) {
        return new RemoteInvocationResult(
            ((InvalidRemoteInvocation) invocation).getException());
      }

      return super.invokeAndCreateResult(invocation, targetObject);
    } catch (Throwable th) {
      log.logError(th);
      return new RemoteInvocationResult(th);
    } finally {
      log.logEnd();
    }
  }

  public void setServiceMethod(String serviceMethod) {
    this.serviceMethod = serviceMethod;
  }

  public void setRequestClass(Class requestClass) {
    this.requestClass = requestClass;
  }

  public void setResponseClass(Class responseClass) {
    this.responseClass = responseClass;
  }
}

/**
 * @author Juan Hall
 */
class InvalidRemoteInvocation extends RemoteInvocation {

  private static final long serialVersionUID = 1L;

  private Throwable exception;

  public Throwable getException() {
    return exception;
  }

  public InvalidRemoteInvocation(final String methodName,
                                 final Throwable exception) {
    this.setMethodName(methodName);
    this.exception = exception;
  }
}