/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.remoting.client;

import com.mockobjects.dynamic.C;
import com.mockobjects.dynamic.Mock;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketAddress;
import javax.net.ssl.SSLContext;
import midstream.framework.LoggingTestCase;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class SocketClientImplTest extends LoggingTestCase {

  @Test
  public void testExecuteWithDelegate() throws Exception {
    Mock delegateMock = new Mock(SocketClientDelegate.class);
    SocketClientImpl impl = new SocketClientImpl() {
      @Override
      byte[] doExecute(SocketClientDelegate delegate, final byte[] bytes) {
        return null;
      }
    };

    impl.execute((SocketClientDelegate) delegateMock.proxy());

    delegateMock.verify();

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }

  @Test
  public void testExecuteWithBytes() throws Exception {
    SocketClientImpl impl = new SocketClientImpl() {
      @Override
      byte[] doExecute(SocketClientDelegate delegate, final byte[] bytes) {
        return null;
      }
    };

    impl.execute("test".getBytes());

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }

  @Test
  public void testDoExecuteWithSecureConnectionAndBytes() throws Exception {
    final MockSocket mockSocket = new MockSocket();

    SocketClientImpl impl = new SocketClientImpl() {
      @Override
      protected Socket createSecureSocket(SSLContext ctx) {
        return mockSocket;
      }
    };
    impl.setHost("http://localhost");
    impl.setSecure(true);
    impl.setSecureProtocol("TLS");
    impl.setKeyAlgorithm("SunX509");
    impl.setKeyStoreType("JKS");
    impl.setKeyStorePassphrase("unit_test");
    impl.setKeyStoreLocation("test/src/" +
        this.getClass().getPackage().getName().replace('.', '/') +
        "/test.keystore");

    assertEquals("response bytes", new String(impl.doExecute(null, "test bytes"
        .getBytes()), "ASCII"));
  }

  @Test
  public void testDoExecuteWithNonSecureConnectionAndDelegate()
      throws Exception {
    final MockSocket mockSocket = new MockSocket();

    SocketClientImpl impl = new SocketClientImpl() {
      @Override
      protected Socket createSocket() {
        return mockSocket;
      }
    };
    impl.setHost("http://localhost");
    impl.setSecure(false);

    Mock delegateMock = new Mock(SocketClientDelegate.class);
    delegateMock.expect("writeRequest", C.isA(OutputStream.class));
    delegateMock.expect("readResponse", C.isA(InputStream.class));
    impl.doExecute((SocketClientDelegate) delegateMock.proxy(), null);

    delegateMock.verify();
  }
}

class MockSocket extends Socket {

  @Override
  public void connect(SocketAddress endpoint, int timeout) throws IOException {
  }

  private ByteArrayInputStream inputStream = new ByteArrayInputStream(
      "response bytes".getBytes());

  @Override
  public InputStream getInputStream() throws IOException {
    return inputStream;
  }

  private ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

  @Override
  public OutputStream getOutputStream() throws IOException {
    return outputStream;
  }
}