/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.remoting.xmlinvoker;

import com.mockobjects.dynamic.C;
import com.mockobjects.dynamic.Mock;
import java.io.Serializable;
import java.io.StringWriter;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import midstream.framework.LoggingTestCase;
import org.jibx.runtime.IBindingFactory;
import org.jibx.runtime.IMarshallingContext;
import org.jibx.runtime.IUnmarshallingContext;
import org.jibx.runtime.JiBXException;
import org.junit.Test;
import org.springframework.remoting.support.RemoteInvocation;
import org.springframework.remoting.support.RemoteInvocationResult;

/**
 * @author Juan Hall
 */
public class XmlHttpInvokerServiceExporterTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSet() {
    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter();
    exporter.setService("test");
    exporter.setServiceInterface(Serializable.class);

    try {
      exporter.afterPropertiesSet();
      fail("An IllegalArgumentException should have been thrown");
    } catch (Exception e) {
      assertTrue(e instanceof IllegalArgumentException);
      assertEquals("A valid service method must be specified", e.getMessage());
    }

    exporter.setServiceMethod("test");

    try {
      exporter.afterPropertiesSet();
      fail("An IllegalArgumentException should have been thrown");
    } catch (Exception e) {
      assertTrue(e instanceof IllegalArgumentException);
      assertEquals("A valid request class must be specified", e.getMessage());
    }

    exporter.setRequestClass(Class.class);

    try {
      exporter.afterPropertiesSet();
      fail("An IllegalArgumentException should have been thrown");
    } catch (Exception e) {
      assertTrue(e instanceof IllegalArgumentException);
      assertEquals("A valid response class must be specified", e.getMessage());
    }

    exporter.setResponseClass(Class.class);

    exporter.afterPropertiesSet();
  }

  @Test
  public void testReadRemoteInvocationHttpServletRequest() throws Exception {
    Object obj = new Object();
    ServletInputStream inputStream = new ServletInputStream() {
      @Override
      public int read() {
        return 0;
      }
    };
    Mock contextMock = new Mock(IUnmarshallingContext.class);
    contextMock.expectAndReturn("unmarshalDocument", C.ANY_ARGS, obj);
    final Mock factoryMock = new Mock(IBindingFactory.class);
    factoryMock
        .expectAndReturn("createUnmarshallingContext", contextMock.proxy());

    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter() {
          @Override
          IBindingFactory getBindingFactory(Class clazz) {
            assertEquals(XmlHttpInvokerServiceExporterTest.class, clazz);
            return (IBindingFactory) factoryMock.proxy();
          }
        };
    exporter.setServiceMethod("method");
    exporter.setRequestClass(this.getClass());

    Mock requestMock = new Mock(HttpServletRequest.class);
    requestMock.expectAndReturn("getInputStream", inputStream);

    RemoteInvocation inv =
        exporter.readRemoteInvocation((HttpServletRequest) requestMock.proxy());

    assertEquals("method", inv.getMethodName());
    assertEquals(this.getClass(), inv.getParameterTypes()[0]);
    assertEquals(obj, inv.getArguments()[0]);

    contextMock.verify();
    factoryMock.verify();
    requestMock.verify();
  }

  @Test
  public void testReadRemoteInvocationWithInvalidInput() throws Exception {
    ServletInputStream inputStream = new ServletInputStream() {
      @Override
      public int read() {
        return 0;
      }
    };
    JiBXException expectedException = new JiBXException(
        "Error parsing document (line 1, col 1)");
    Mock contextMock = new Mock(IUnmarshallingContext.class);
    contextMock
        .expectAndThrow("unmarshalDocument", C.ANY_ARGS, expectedException);
    final Mock factoryMock = new Mock(IBindingFactory.class);
    factoryMock
        .expectAndReturn("createUnmarshallingContext", contextMock.proxy());

    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter() {
          @Override
          IBindingFactory getBindingFactory(Class clazz) {
            assertEquals(XmlHttpInvokerServiceExporterTest.class, clazz);
            return (IBindingFactory) factoryMock.proxy();
          }
        };
    exporter.setServiceMethod("method");
    exporter.setRequestClass(this.getClass());

    Mock requestMock = new Mock(HttpServletRequest.class);
    requestMock.expectAndReturn("getInputStream", inputStream);

    RemoteInvocation inv =
        exporter.readRemoteInvocation((HttpServletRequest) requestMock.proxy());

    assertTrue(inv instanceof InvalidRemoteInvocation);
    assertEquals(expectedException, ((InvalidRemoteInvocation) inv)
        .getException());

    contextMock.verify();
    factoryMock.verify();
    requestMock.verify();
  }

  @Test
  public void testWriteRemoteInvocationResult() throws Exception {
    Object obj = new Object();
    ServletOutputStream outputStream = new ServletOutputStream() {
      @Override
      public void write(int arg0) {
      }
    };

    Mock contextMock = new Mock(IMarshallingContext.class);
    contextMock.expect("marshalDocument", C.ANY_ARGS);

    final Mock factoryMock = new Mock(IBindingFactory.class);
    factoryMock
        .expectAndReturn("createMarshallingContext", contextMock.proxy());

    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter() {
          @Override
          IBindingFactory getBindingFactory(Class clazz) {
            assertEquals(XmlHttpInvokerServiceExporterTest.class, clazz);
            return (IBindingFactory) factoryMock.proxy();
          }
        };
    exporter.setServiceMethod("method");
    exporter.setResponseClass(this.getClass());

    Mock responseMock = new Mock(HttpServletResponse.class);
    responseMock.expectAndReturn("getOutputStream", outputStream);

    RemoteInvocationResult result = new RemoteInvocationResult(obj);
    exporter.writeRemoteInvocationResult(null,
        (HttpServletResponse) responseMock.proxy(), result);

    contextMock.verify();
    factoryMock.verify();
    responseMock.verify();
  }

  @Test
  public void testWriteRemoteInvocationResultWithException() throws Exception {
    final Throwable expectedException = new Exception();
    final boolean[] checks = {false};

    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter() {
          @Override
          void outputExceptionResponse(HttpServletResponse response,
                                       Throwable e) {
            checks[0] = true;
            assertEquals(expectedException, e);
          }
        };

    RemoteInvocationResult result = new RemoteInvocationResult(
        expectedException);
    exporter.writeRemoteInvocationResult(null, null, result);

    assertTrue(checks[0]);
  }

  @Test
  public void testWriteRemoteInvocationResultException() throws Exception {
    Object obj = new Object();
    ServletOutputStream outputStream = new ServletOutputStream() {
      @Override
      public void write(int arg0) {
      }
    };

    Mock contextMock = new Mock(IMarshallingContext.class);
    final Throwable expectedException = new RuntimeException("error");
    contextMock
        .expectAndThrow("marshalDocument", C.ANY_ARGS, expectedException);
    final Mock factoryMock = new Mock(IBindingFactory.class);
    factoryMock
        .expectAndReturn("createMarshallingContext", contextMock.proxy());

    final boolean[] checks = {false};

    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter() {
          @Override
          IBindingFactory getBindingFactory(Class clazz) {
            assertEquals(XmlHttpInvokerServiceExporterTest.class, clazz);
            return (IBindingFactory) factoryMock.proxy();
          }

          @Override
          void outputExceptionResponse(HttpServletResponse response,
                                       Throwable e) {
            checks[0] = true;
            assertEquals(expectedException, e);
          }
        };
    exporter.setServiceMethod("method");
    exporter.setResponseClass(this.getClass());

    Mock responseMock = new Mock(HttpServletResponse.class);
    responseMock.expectAndReturn("getOutputStream", outputStream);

    exporter.writeRemoteInvocationResult(null,
        (HttpServletResponse) responseMock.proxy(),
        new RemoteInvocationResult(obj));

    assertTrue(checks[0]);

    contextMock.verify();
    factoryMock.verify();
    responseMock.verify();
  }

  @Test
  public void testOutputExceptionResponse() throws Exception {
    final StringWriter writer = new StringWriter();
    ServletOutputStream outputStream = new ServletOutputStream() {
      @Override
      public void write(int arg0) {
        writer.write(arg0);
      }
    };

    Mock responseMock = new Mock(HttpServletResponse.class);
    responseMock.expectAndReturn("getOutputStream", outputStream);

    new XmlHttpInvokerServiceExporter()
        .outputExceptionResponse((HttpServletResponse) responseMock.proxy(),
            new Exception("error"));

    String xmlResult = writer.getBuffer().toString();
    assertTrue(xmlResult
        .startsWith("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
    assertTrue(xmlResult.contains("<type>java.lang.Exception</type>"));
    assertTrue(xmlResult.contains("<message>error</message>"));

    responseMock.verify();
  }

  @Test
  public void testInvokeAndCreateResultRemoteInvocationObject()
      throws Exception {
    String target = "target";
    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter();
    exporter.setServiceInterface(Serializable.class);

    RemoteInvocation invocation = new RemoteInvocation("toString", null, null);
    exporter.invokeAndCreateResult(invocation, target);

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }

  @Test
  public void testInvokeAndCreateResultWithInvalidInput() throws Exception {
    String target = "target";
    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter();
    exporter.setServiceInterface(Serializable.class);

    InvalidRemoteInvocation invocation = new InvalidRemoteInvocation(
        "toString", new Exception("error"));
    RemoteInvocationResult result = exporter.invokeAndCreateResult(invocation,
        target);

    assertEquals("error", result.getException().getMessage());
  }

  @Test
  public void testInvokeAndCreateResultWithInvocationException()
      throws Exception {
    String target = "target";
    XmlHttpInvokerServiceExporter exporter =
        new XmlHttpInvokerServiceExporter();
    exporter.setServiceInterface(Serializable.class);

    InvalidRemoteInvocation invocation = new InvalidRemoteInvocation("bar",
        null) {
      private static final long serialVersionUID = 1L;

      @Override
      public Throwable getException() {
        throw new RuntimeException("error");
      }

      @Override
      public String toString() {
        return InvalidRemoteInvocation.class.getName();
      }
    };

    exporter.invokeAndCreateResult(invocation, target);

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }
}
