/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core;

import bsh.EvalError;
import bsh.Interpreter;
import com.thoughtworks.xstream.XStream;
import midstream.core.handler.MdsTestHandler;
import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class MdsTestRunner implements Runnable {

  private static final Map<String, Class> PRIMITIVES_MAP =
      new HashMap<String, Class>(
          8);

  static {
    PRIMITIVES_MAP.put("boolean", Boolean.TYPE);
    PRIMITIVES_MAP.put("byte", Byte.TYPE);
    PRIMITIVES_MAP.put("char", Character.TYPE);
    PRIMITIVES_MAP.put("double", Double.TYPE);
    PRIMITIVES_MAP.put("float", Float.TYPE);
    PRIMITIVES_MAP.put("int", Integer.TYPE);
    PRIMITIVES_MAP.put("long", Long.TYPE);
    PRIMITIVES_MAP.put("short", Short.TYPE);
  }

  private ConsoleOperation consoleOperation;

  private Map<String, String> formParams;

  public MdsTestRunner(ConsoleOperation operation, Map<String, String> params) {
    this.consoleOperation = operation;
    this.formParams = params;
  }

  public void run() {
    ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();

    try {
      new File(formParams.get("project.path") + File.separatorChar + "build"
          + File.separatorChar + "META-INF").mkdirs();
      try {
        WizardUtils.copyFile(formParams.get("project.path")
            + File.separatorChar + "conf" + File.separatorChar
            + "clientContext.xml", formParams.get("project.path")
            + File.separatorChar + "build" + File.separatorChar + "META-INF"
            + File.separatorChar + "clientContext.xml", true);
      } catch (IOException e) {
        e.printStackTrace();
      }
      URLClassLoader newClassLoader =
          new MdsTestHandler(formParams.get("project.path"))
              .createMdsTestClassLoader(formParams.get("project.path")
                  + File.separatorChar + "lib", "true".equals(formParams
                  .get("use.client.jar")));
      Thread.currentThread().setContextClassLoader(newClassLoader);
      Document document = WizardUtils.readXmlDocument(new File(formParams
          .get("mds.tests.path")
          + File.separatorChar + formParams.get("test.name") + ".xml"), true);

      if (document.selectSingleNode("/mds/param-test") != null) {
        runJavaHttpTest(document, newClassLoader);
      } else if (document.selectSingleNode("/mds/post-test") != null) {
        runPostHttpTest(document);
      } else {
        runFreeFormTest(document);
      }
    } catch (Exception e) {
      consoleOperation.printlnError("Exception:");
      consoleOperation.printlnError(new XStream().toXML(e));
      consoleOperation.showErrorMessage("Error running test: "
          + formParams.get("test.name") + ".\nReview console for details.");
    } finally {
      Thread.currentThread().setContextClassLoader(oldClassLoader);
      consoleOperation.dispose();
    }
  }

  private void runFreeFormTest(Document document) throws
      UnsupportedEncodingException {
    Object result;
    PrintStream normalOut = System.out;
    PrintStream normalErr = System.err;

    try {
      ByteArrayOutputStream outStream = new ByteArrayOutputStream();
      ByteArrayOutputStream errStream = new ByteArrayOutputStream();

      System.setOut(new PrintStream(outStream));
      System.setErr(new PrintStream(errStream));

      Object scriptResult = new Interpreter().eval(document
          .selectSingleNode("//script").getText().trim());

      result = new Object[]{
          new Object[]{"Script Result", scriptResult},
          new String[]{"STD Output", outStream.toString("UTF-8")},
          new String[]{"STD Error", errStream.toString("UTF-8")}};
    } catch (EvalError evalError) {
      result = evalError;
    } finally {
      System.setOut(normalOut);
      System.setErr(normalErr);
    }

    consoleOperation.printlnHeader("Output:");
    consoleOperation.printValue(new XStream().toXML(result));
  }

  private void runPostHttpTest(Document document) {
    String result;
    consoleOperation.printHeader("Calling Service: ");
    consoleOperation.printlnValue(document
        .selectSingleNode("//post-test/@url").getText().trim());

    consoleOperation.printlnHeader(null);
    consoleOperation.printlnHeader("Payload");
    consoleOperation.printlnValue(document
        .selectSingleNode("//payload").getText().trim());

    StringBuffer buf = new StringBuffer();

    try {
      URL url =
          new URL(document
              .selectSingleNode("//post-test/@url").getText().trim());
      URLConnection conn = url.openConnection();
      conn.setDoOutput(true);

      OutputStreamWriter wr = null;

      try {
        wr =
            new OutputStreamWriter(conn.getOutputStream());
        wr.write(document
            .selectSingleNode("//payload").getText().trim());
        wr.flush();
      } finally {
        if (wr != null) {
          wr.close();
        }
      }

      BufferedReader rd = null;

      try {
        rd = new BufferedReader(
            new InputStreamReader(conn.getInputStream()));
        String line;
        while ((line = rd.readLine()) != null) {
          buf.append(line);
        }

        result = buf.toString();
      } finally {
        if (rd != null) {
          rd.close();
        }
      }

      try {
        Transformer serializer =
            TransformerFactory.newInstance().newTransformer();
        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
        serializer.setOutputProperty(
            "{http://xml.apache.org/xslt}indent-amount", "2");

        StringWriter sr = new StringWriter();
        serializer.transform(new StreamSource(new StringReader(result)),
            new StreamResult(sr));

        result = sr.getBuffer().toString();
      } catch (TransformerException e) {
        // Ignore formatting errors
      }
    } catch (Exception e) {
      result = new XStream().toXML(e);
    }

    consoleOperation.printlnHeader(null);
    consoleOperation.printlnHeader("Response:");
    consoleOperation.printValue(result);
  }

  private void runJavaHttpTest(Document document,
                               URLClassLoader newClassLoader) throws
      ClassNotFoundException, NoSuchMethodException, IllegalAccessException,
      InvocationTargetException {
    Object result;
    List nodes = document.selectNodes("//param");
    List<Object> values = new ArrayList<Object>(nodes.size());
    List<Class> types = new ArrayList<Class>(nodes.size());
    Interpreter shell = new Interpreter();

    for (Object node : nodes) {
      Element elem = (Element) node;
      try {
        if (!PRIMITIVES_MAP.containsKey(elem.attributeValue("type"))) {
          types.add(newClassLoader.loadClass(elem.attributeValue("type")));
        } else {
          types.add(PRIMITIVES_MAP.get(elem.attributeValue("type")));
        }

        values.add(shell.eval(elem.getTextTrim()));
      } catch (EvalError evalError) {
        evalError.printStackTrace();
      }
    }
    String serviceName = document.selectSingleNode("//param-test/@service")
        .getText();
    Class<?> serviceLocatorClass = newClassLoader
        .loadClass("midstream.framework.remoting.client.ServiceLocator");
    Class clazz = newClassLoader.loadClass(serviceName);
    Method method = clazz.getDeclaredMethod(document.selectSingleNode(
        "//param-test/@method").getText(), types.toArray(new Class[0]));
    String serviceUrl = serviceLocatorClass.getDeclaredMethod(
        "getServiceUrl", String.class).invoke(null, serviceName).toString();
    serviceUrl = serviceUrl.replaceFirst("//\\w+:\\d+", "//"
        + formParams.get("host") + ":" + formParams.get("port"));
    serviceLocatorClass.getDeclaredMethod("setServiceUrl", String.class,
        String.class).invoke(null, serviceName, serviceUrl);

    Object service = serviceLocatorClass.getMethod("getService",
        String.class).invoke(null, serviceName);

    consoleOperation.printHeader("Calling Service: ");
    consoleOperation.printlnValue(serviceUrl);

    consoleOperation.printlnHeader(null);
    consoleOperation.printlnHeader("Parameters:");
    consoleOperation.printlnValue(new XStream().toXML(values
        .toArray(new Object[0])));

    try {
      result = method.invoke(service, values.toArray(new Object[0]));
    } catch (Exception e) {
      result = e;
    }

    consoleOperation.printlnHeader(null);
    consoleOperation.printlnHeader("Response:");
    consoleOperation.printValue(new XStream().toXML(result));
  }
}