/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.WizardUtils;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.Element;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class AddBeanPropertyHandler extends BaseHandler {

  public static final int REFERENCE = 0;

  public static final int VALUE = 1;

  public static final int COLLECTION = 2;

  private static final List<String> STANDARD_CLASSES =
      Arrays.asList(new String[]{
          "org.apache.commons.configuration.Configuration",
          "net.sf.ehcache.CacheManager"});

  public AddBeanPropertyHandler(String projectPath) {
    super(projectPath);
  }

  public String addBeanProperty(
      Map<String, Object> pageValues,
      String beanName) throws Exception {
    Document doc = readApplicationContext();

    Integer index = (Integer) pageValues.get("property.tab.index");

    Element bean = (Element) doc.getRootElement().selectSingleNode(
        "//ns:bean[@id='" + beanName + "' or @name='" + beanName + "']");

    if (bean == null) {
      throw new Exception("Unable to resolve bean: " + beanName);
    }

    final String propertyName = (String) pageValues.get("property.name");
    if (bean.selectSingleNode("./ns:property[@name='" + propertyName + "']") !=
        null) {
      throw new Exception("The property '" + propertyName
          + "' already exists on bean: " + beanName);
    }

    String srcFile = null;

    switch (index) {
      case REFERENCE:
        srcFile = createReference(pageValues, bean);
        break;
      case VALUE:
        srcFile = createValue(pageValues, bean);
        break;
      case COLLECTION:
        srcFile = createCollection(pageValues, bean);
        break;
    }

    outputApplicationContext(doc);

    return srcFile;
  }

  public List<String> buildClassNameList() {
    ClassLoader currentClassLoader = Thread.currentThread()
        .getContextClassLoader();
    final List<String> classes = new ArrayList<String>(5);

    try {
      classes.addAll(STANDARD_CLASSES);

      String contents = readFile(projectPath + File.separatorChar + "conf"
          + File.separatorChar + "applicationContext.xml");
      contents = contents.replaceAll("\\$\\{CONF_DIR\\}", "conf");
      writeFile(
          projectPath + File.separatorChar + "build" + File.separatorChar +
              "applicationContext.xml", contents, true);

      List<URL> urls =
          new ProjectHandler(projectPath).getProjectClassUrls(projectPath
              + File.separatorChar + "lib", false);

      List<URL> excludes = new ArrayList<URL>(3);

      for (URL url : urls) {
        if (url.getPath().contains("xerces")) {
          excludes.add(url);
        }
      }

      urls.removeAll(excludes);

      URLClassLoader projectClassLoader = new URLClassLoader(urls
          .toArray(new URL[0]), currentClassLoader.getParent()) {
        @Override
        protected Class<?> findClass(String name)
            throws ClassNotFoundException {
          if (!classes.contains(name)) {
            classes.add(name);
          }

          return super.findClass(name);
        }
      };

      Thread.currentThread().setContextClassLoader(projectClassLoader);

      System.setProperty("MIDSTREAM_ENV", "Test");

      Class
          .forName(
              "org.springframework.context.support.ClassPathXmlApplicationContext",
              true, projectClassLoader).getConstructor(
          String.class).newInstance("classpath*:/applicationContext.xml");
    } catch (Exception ex) {
      ex.printStackTrace();
    } finally {
      Thread.currentThread().setContextClassLoader(currentClassLoader);

      System.clearProperty("MIDSTREAM_ENV");
    }

    return classes;
  }

  public String[] getMatchingItems(String string, List<String> classes) {
    if (string == null) {
      return new String[0];
    }

    List<String> options = new ArrayList<String>(4);

    for (String name : classes) {
      if (name != null && name.toLowerCase().contains(string.toLowerCase())) {
        options.add(name);
      }
    }

    Collections.sort(options);

    return options.toArray(new String[0]);
  }

  public List<String> getContextBeanNames() {
    Document doc;

    try {
      doc = readApplicationContext();
    } catch (IOException e) {
      e.printStackTrace();
      doc = null;
    }

    if (doc == null) {
      return null;
    }

    List<String> names = new ArrayList<String>(5);
    for (Object name : doc.getRootElement().selectNodes(
        "//ns:bean/@name | //ns:bean/@id")) {
      names.add(((Attribute) name).getText());
    }

    Collections.sort(names);

    return names;
  }

  String createCollection(
      Map<String, Object> pageValues,
      Element bean) {
    String srcFile = null;

    if ("Properties".equals(pageValues.get("collection.type"))) {
      srcFile = createPropertiesProperty(pageValues, bean);
    } else if ("Map".equals(pageValues.get("collection.type"))) {
      srcFile = createMapProperty(pageValues, bean);
    } else if ("List".equals(pageValues.get("collection.type"))) {
      srcFile = createListProperty(pageValues, bean);
    } else if ("Set".equals(pageValues.get("collection.type"))) {
      srcFile = createSetProperty(pageValues, bean);
    }

    return srcFile;
  }

  String createPropertiesProperty(
      Map<String, Object> pageValues,
      Element bean) {
    List<String[]> collection = (List<String[]>) pageValues.get("collection");
    Element propsElem = bean.addElement("property").addAttribute("name",
        (String) pageValues.get("property.name"));

    getBeanHandler()
        .addPropertiesToPropertyElement(propsElem, collection);

    String srcFile = null;

    if ((Boolean) pageValues.get("add.getter.setter")) {
      srcFile = "src" + File.separatorChar
          + bean.attributeValue("class").replace('.', File.separatorChar)
          + ".java";
      addGetterSetter(srcFile, "java.util.Properties",
          (String) pageValues.get("property.name"));
    }

    return srcFile;
  }

  protected BeanHandler getBeanHandler() {
    return new BeanHandler(projectPath);
  }

  String createMapProperty(
      Map<String, Object> pageValues,
      Element bean) {
    List<String[]> collection = (List<String[]>) pageValues.get("collection");
    Element mapElem = bean.addElement("property").addAttribute("name",
        (String) pageValues.get("property.name"));

    getBeanHandler().addMapToPropertyElement(mapElem, collection);

    String srcFile = null;

    if ((Boolean) pageValues.get("add.getter.setter")) {
      srcFile = "src" + File.separatorChar
          + bean.attributeValue("class").replace('.', File.separatorChar)
          + ".java";
      addGetterSetter(srcFile, "java.util.Map",
          (String) pageValues.get("property.name"));
    }

    return srcFile;
  }

  String createListProperty(
      Map<String, Object> pageValues,
      Element bean) {
    List<String[]> collection = (List<String[]>) pageValues.get("collection");
    Element listElem = bean.addElement("property").addAttribute("name",
        (String) pageValues.get("property.name"));

    getBeanHandler().addListToPropertyElement(listElem, collection);

    String srcFile = null;

    if ((Boolean) pageValues.get("add.getter.setter")) {
      srcFile = "src" + File.separatorChar
          + bean.attributeValue("class").replace('.', File.separatorChar)
          + ".java";
      addGetterSetter(srcFile, "java.util.List",
          (String) pageValues.get("property.name"));
    }

    return srcFile;
  }

  String createSetProperty(
      Map<String, Object> pageValues,
      Element bean) {
    List<String[]> collection = (List<String[]>) pageValues.get("collection");
    Element setElem = bean.addElement("property").addAttribute("name",
        (String) pageValues.get("property.name"));

    getBeanHandler().addSetToPropertyElement(setElem, collection);

    String srcFile = null;

    if ((Boolean) pageValues.get("add.getter.setter")) {
      srcFile = "src" + File.separatorChar
          + bean.attributeValue("class").replace('.', File.separatorChar)
          + ".java";
      addGetterSetter(srcFile, "java.util.Set",
          (String) pageValues.get("property.name"));
    }

    return srcFile;
  }

  String createValue(
      Map<String, Object> pageValues,
      Element bean) {
    bean.addElement("property").addAttribute("name",
        (String) pageValues.get("property.name")).addAttribute("value",
        (String) pageValues.get("value"));

    String srcFile = null;

    if ((Boolean) pageValues.get("add.getter.setter")) {
      srcFile = "src" + File.separatorChar
          + bean.attributeValue("class").replace('.', File.separatorChar)
          + ".java";
      addGetterSetter(srcFile, (String) pageValues
          .get("value.type"), (String) pageValues.get("property.name"));
    }

    return srcFile;
  }

  String createReference(
      Map<String, Object> pageValues,
      Element bean) {
    bean.addElement("property").addAttribute("name",
        (String) pageValues.get("property.name")).addAttribute("ref",
        (String) pageValues.get("reference.bean"));

    String srcFile = null;

    if ((Boolean) pageValues.get("add.getter.setter")) {
      srcFile = "src" + File.separatorChar
          + bean.attributeValue("class").replace('.', File.separatorChar)
          + ".java";
      addGetterSetter(srcFile,
          (String) pageValues.get("fqn.type"), (String) pageValues
          .get("property.name"));
    }

    return srcFile;
  }

  void addGetterSetter(String srcFile, String type,
                       String name) {
    try {
      String template = WizardUtils.readStream(AddBeanPropertyHandler.class
          .getClassLoader().getResourceAsStream(
          "midstream/core/templates/components/getter_setter.template"));

      String contents = readFile(projectPath + File.separatorChar + srcFile);

      if (contents != null && contents.length() > 0) {
        String prop = template.replaceAll("@TYPE@", type).replaceAll("@PROP@",
            name).replaceAll("@METHOD@", StringUtils.capitalize(name));
        contents = new StringBuffer(contents).insert(contents.indexOf('{') + 1,
            prop).toString();

        writeFile(projectPath + File.separatorChar + srcFile,
            contents, true);
      }
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
}