/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import com.thoughtworks.xstream.XStream;
import org.dom4j.Document;
import org.dom4j.Element;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * @author Juan Hall
 */
public class BeanHandler extends BaseHandler {

  public BeanHandler(String projectPath) {
    super(projectPath);
  }

  public void createBean(String name, String clazz,
                         Map<String, String> props) throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc.selectSingleNode("//ns:bean[@id='" + name + "']") != null) {
      return;
    }

    Element bean = root.addElement("bean").addAttribute("id", name)
        .addAttribute("class", clazz);

    for (Map.Entry<String, String> entry : props.entrySet()) {
      Element property = bean.addElement("property").addAttribute("name",
          entry.getKey());

      if (entry.getValue().startsWith("ref=")) {
        property.addAttribute("ref", entry.getValue()
            .substring("ref=".length()));
      } else if (Pattern.compile("^(Properties|List|Map|Set)=.*",
          Pattern.MULTILINE).matcher(entry.getValue()).find()) {
        List<String[]> collection = (List<String[]>) new XStream()
            .fromXML(entry.getValue().substring(
                entry.getValue().indexOf('=') + 1));

        if (entry.getValue().startsWith("Properties")) {
          addPropertiesToPropertyElement(property, collection);
        } else if (entry.getValue().startsWith("List")) {
          addListToPropertyElement(property, collection);
        } else if (entry.getValue().startsWith("Map")) {
          addMapToPropertyElement(property, collection);
        } else if (entry.getValue().startsWith("Set")) {
          addSetToPropertyElement(property, collection);
        }
      } else {
        property.addAttribute("value", entry.getValue());
      }
    }

    outputApplicationContext(doc);
  }

  void addPropertiesToPropertyElement(Element property,
                                             List<String[]> collection) {
    Element props = property.addElement("props");

    for (String[] item : collection) {
      if (!"".equals(item[1].trim())) {
        props.addElement("prop").addAttribute("key", item[1]).addText(item[2]);
      }
    }
  }

  void addListToPropertyElement(Element property,
                                       List<String[]> collection) {
    Element list = property.addElement("list");

    for (String[] item : collection) {
      if ("".equals(item[2].trim())) {
        continue;
      }

      if ("true".equals(item[0])) {
        list.addElement("ref").addAttribute("bean", item[2]);
      } else {
        list.addElement("value").addText(item[2]);
      }
    }
  }

  void addMapToPropertyElement(Element property,
                                      List<String[]> collection) {
    Element map = property.addElement("map");

    for (String[] item : collection) {
      if (("true".equals(item[0]) && "".equals(item[2].trim()))
          || "".equals(item[1].trim())) {
        continue;
      }

      Element entryElem = map.addElement("entry");
      entryElem.addElement("key").addElement("value").addText(item[1]);

      if ("true".equals(item[0])) {
        entryElem.addElement("ref").addAttribute("bean", item[2]);
      } else {
        entryElem.addElement("value").addText(item[2]);
      }
    }
  }

  void addSetToPropertyElement(Element property,
                                      List<String[]> collection) {
    Element set = property.addElement("set");

    for (String[] item : collection) {
      if ("".equals(item[2].trim())) {
        continue;
      }

      if ("true".equals(item[0])) {
        set.addElement("ref").addAttribute("bean", item[2]);
      } else {
        set.addElement("value").addText(item[2]);
      }
    }
  }
}
