/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.SourceUtils;
import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.springframework.util.FileCopyUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * @author Juan Hall
 */
public class JmsActorHandler extends BaseHandler {

  public static final String[] LIBRARIES =
      {"jms.jar", "cglib:cglib-nodep", "mockobjects:mockobjects-core"};

  public JmsActorHandler(String projectPath) {
    super(projectPath);
  }

  enum PropertiesType {
    Override, Placeholder
  }

  public String createReceiverClass(
      Map<String, String> pageValues)
      throws IOException {
    if (!new File(
        projectPath + File.separatorChar + "lib" + File.separatorChar +
            "jms.jar").exists()) {
      FileCopyUtils
          .copy(ProjectHandler.class.getClassLoader().getResourceAsStream(
              "midstream/core/resources/lib/jms.jar"),
              new FileOutputStream(
                  projectPath + File.separatorChar + "lib" + File
                      .separatorChar +
                      "jms.jar"));
    }

    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("actor.name"));

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("actor.name"), stubs, "JmsReceiverClass");
  }

  public void createReceiverClassTest(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("actor.name"));

    SourceUtils.createSourceClassTest(projectPath,
        pageValues.get("actor.name"), stubs, "JmsReceiverClass");
  }

  public String createProducerClass(
      Map<String, String> pageValues)
      throws IOException {
    if (!new File(
        projectPath + File.separatorChar + "lib" + File.separatorChar +
            "jms.jar").exists()) {
      FileCopyUtils
          .copy(ProjectHandler.class.getClassLoader().getResourceAsStream(
              "midstream/core/resources/lib/jms.jar"),
              new FileOutputStream(
                  projectPath + File.separatorChar + "lib" + File
                      .separatorChar +
                      "jms.jar"));
    }

    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("actor.name"));

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("actor.name"), stubs, "JmsProducerClass");
  }

  public void createProducerClassTest(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("actor.name"));

    SourceUtils.createSourceClassTest(projectPath,
        pageValues.get("actor.name"), stubs, "JmsProducerClass");
  }

  public void createReceiverConfiguration(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    addReceiverTargetBean(pageValues, doc, root);

    addReceiverBean(pageValues, doc, root);

    addCredentialsConnectionFactory(pageValues, doc, root);

    addConnectionFactoryBean(pageValues, doc, root);

    addJndiTemplate(pageValues, doc, root);

    addJndiDestinationBean(pageValues, doc, root);

    createEnvironmentProperties(pageValues);

    addPlaceholderConfig(doc, root);

    addPropertyOverrideConfig(doc, root);

    outputApplicationContext(doc);
  }

  void addPlaceholderConfig(Document doc,
                            Element root) {
    Element node = WizardUtils.addPlaceholderConfig(doc, root);

    if (null != node) {
      node
          .addElement("property")
          .addAttribute("name", "location")
          .addAttribute(
              "value",
              "${CONF_DIR}/placeholder.properties.${MIDSTREAM_ENV}");
    }
  }

  public void createProducerConfiguration(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    addProducerBean(pageValues, doc, root);

    addProducerTemplateBeans(pageValues, doc, root);

    addConnectionFactoryBean(pageValues, doc, root);

    addJndiTemplate(pageValues, doc, root);

    addJndiDestinationBean(pageValues, doc, root);

    createEnvironmentProperties(pageValues);

    addPlaceholderConfig(doc, root);

    addPropertyOverrideConfig(doc, root);

    outputApplicationContext(doc);
  }

  void addProducerTemplateBeans(Map<String, String> pageValues,
                                Document doc, Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("actor.name")
        + "TemplateTarget']") == null) {
      Element bean = root.addElement("bean");
      bean
          .addAttribute("id", pageValues.get("actor.name") + "TemplateTarget")
          .addAttribute("class", "org.springframework.jms.core.JmsTemplate102");

      if ("".equals(pageValues.get("user.name"))) {
        bean
            .addElement("property")
            .addAttribute("name", "connectionFactory")
            .addAttribute(
                "ref",
                "true".equals(pageValues.get("is.queue")) ?
                    "QueueConnectionFactory"
                    : "TopicConnectionFactory");
      } else {
        bean
            .addElement("property")
            .addAttribute("name", "connectionFactory")
            .addAttribute(
                "ref",
                "true".equals(pageValues.get("is.queue")) ?
                    "CredentialsQueueConnectionFactory"
                    : "CredentialsTopicConnectionFactory");
      }

      if (!"true".equals(pageValues.get("is.queue"))) {
        bean.addElement("property").addAttribute("name", "pubSubDomain")
            .addAttribute("value", "true");
      }

      bean.addElement("property").addAttribute("name", "defaultDestination")
          .addAttribute(
              "ref",
              pageValues.get("actor.name")
                  + ("true".equals(pageValues.get("is.queue")) ? "Queue"
                  : "Topic"));
    }

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("actor.name")
        + "Template']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id", pageValues.get("actor.name") + "Template")
          .addAttribute("class",
              "midstream.framework.jms.producer.JmsTemplateProxyFactoryBean");
      bean.addElement("property").addAttribute("name", "target").addAttribute(
          "ref", pageValues.get("actor.name") + "TemplateTarget");
    }

    addCredentialsConnectionFactory(pageValues, doc, root);
  }

  void addCredentialsConnectionFactory(
      Map<String, String> pageValues, Document doc, Element root) {
    if (!"".equals(pageValues.get("user.name"))) {
      if (doc
          .selectSingleNode("//bean[@id='"
              + ("true".equals(pageValues.get("is.queue")) ?
              "CredentialsQueueConnectionFactory"
              : "CredentialsTopicConnectionFactory") + "']") == null) {

        Element bean = root.addElement("bean");
        bean
            .addAttribute(
                "id",
                "true".equals(pageValues.get("is.queue")) ?
                    "CredentialsQueueConnectionFactory"
                    : "CredentialsTopicConnectionFactory")
            .addAttribute("class",
                "org.springframework.jms.connection.UserCredentialsConnectionFactoryAdapter");
        bean
            .addElement("property")
            .addAttribute("name", "targetConnectionFactory")
            .addAttribute(
                "ref",
                "true".equals(pageValues.get("is.queue")) ?
                    "QueueConnectionFactory"
                    : "TopicConnectionFactory");
      }
    }
  }

  void addJndiDestinationBean(Map<String, String> pageValues,
                              Document doc, Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("actor.name")
        + ("true".equals(pageValues.get("is.queue")) ? "Queue" : "Topic")
        + "']") == null) {
      Element bean = root.addElement("bean");
      bean
          .addAttribute(
              "id",
              pageValues.get("actor.name")
                  + ("true".equals(pageValues.get("is.queue")) ? "Queue"
                  : "Topic")).addAttribute("class",
          "org.springframework.jndi.JndiObjectFactoryBean");
      bean.addElement("property").addAttribute("name", "jndiTemplate")
          .addAttribute(
              "ref",
              "true".equals(pageValues.get("is.queue")) ? "queueJndiTemplate"
                  : "topicJndiTemplate");
    }
  }

  void addReceiverTargetBean(Map<String, String> pageValues,
                             Document doc, Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("actor.name")
        + "Target" + "']") == null) {
      root.addElement("bean").addAttribute("id",
          pageValues.get("actor.name") + "Target").addAttribute("class",
          pageValues.get("java.package") + "." + pageValues.get("actor.name"));
    }
  }

  void addProducerBean(Map<String, String> pageValues,
                       Document doc, Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("java.package")
        + "." + pageValues.get("actor.name") + "']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id",
          pageValues.get("java.package") + "." + pageValues.get("actor.name"))
          .addAttribute(
              "class",
              pageValues.get("java.package") + "."
                  + pageValues.get("actor.name"));
      bean.addElement("property").addAttribute("name", "jmsTemplate")
          .addAttribute("ref", pageValues.get("actor.name") + "Template");
    }
  }

  void addConnectionFactoryBean(Map<String, String> pageValues,
                                Document doc, Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='"
        + ("true".equals(pageValues.get("is.queue")) ? "QueueConnectionFactory"
        : "TopicConnectionFactory") + "']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id",
          "true".equals(pageValues.get("is.queue")) ? "QueueConnectionFactory"
              : "TopicConnectionFactory");
      bean.addAttribute("class",
          "org.springframework.jndi.JndiObjectFactoryBean").addElement(
          "property").addAttribute("name", "jndiTemplate").addAttribute(
          "ref",
          "true".equals(pageValues.get("is.queue")) ? "queueJndiTemplate"
              : "topicJndiTemplate");
    }
  }

  void addReceiverBean(Map<String, String> pageValues, Document doc,
                       Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("actor.name")
        + "']") == null) {
      Element bean = root.addElement("bean");
      bean
          .addAttribute("id", pageValues.get("actor.name"))
          .addAttribute(
              "class",
              "midstream.framework.jms.listener.DefaultMessageListenerContainer");
      bean.addElement("property").addAttribute("name",
          "concurrentConsumers")
          .addAttribute("value", pageValues.get("listener.count"));

      Element property = bean
          .addElement("property")
          .addAttribute("name", "connectionFactory");

      if (!"".equals(pageValues.get("user.name"))) {
        property.addAttribute(
            "ref",
            "true".equals(pageValues.get("is.queue")) ?
                "CredentialsQueueConnectionFactory"
                : "CredentialsTopicConnectionFactory");
      } else {
        property.addAttribute(
            "ref",
            "true".equals(pageValues.get("is.queue")) ?
                "QueueConnectionFactory"
                : "TopicConnectionFactory");
      }

      bean.addElement("property").addAttribute("name",
          "destination")
          .addAttribute(
              "ref",
              pageValues.get("actor.name")
                  + ("true".equals(pageValues.get("is.queue")) ? "Queue"
                  : "Topic"));

      bean.addElement("property").addAttribute("name", "messageListener")
          .addAttribute("ref", pageValues.get("actor.name") + "Target");
    }
  }

  void addPropertyOverrideConfig(Document doc,
                                 Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='propertyOverrideConfig']") == null)
    {
      root
          .addElement("bean")
          .addAttribute("id", "propertyOverrideConfig")
          .addAttribute("class",
              "org.springframework.beans.factory.config.PropertyOverrideConfigurer")
          .addElement("property").addAttribute("name", "location")
          .addAttribute(
              "value",
              "${CONF_DIR}/override.properties.${MIDSTREAM_ENV}");
    }
  }

  void addJndiTemplate(Map<String, String> pageValues, Document doc,
                       Element root) {
    String beanId =
        ("true".equals(pageValues.get("is.queue")) ? "queueJndiTemplate"
            : "topicJndiTemplate");
    if (doc.selectSingleNode("//ns:bean[@id='" + beanId + "']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id", beanId).addAttribute("class",
          "org.springframework.jndi.JndiTemplate").addAttribute("lazy-init",
          "true");
      Element property = bean.addElement("property").addAttribute("name",
          "environment");
      Element props = property.addElement("props");
      props.addElement("prop").addAttribute("key",
          "java.naming.factory.initial").addText(
          pageValues.get("naming.factory.initial"));
      props.addElement("prop").addAttribute("key", "java.naming.provider.url")
          .addText("${" + beanId + ".provider.url}");
      props.addElement("prop").addAttribute("key",
          "java.naming.security.principal").addText(
          "${" + beanId + ".principal}");
      props.addElement("prop").addAttribute("key",
          "java.naming.security.credentials").addText(
          "${" + beanId + ".credentials}");
    }
  }

  void createEnvironmentProperties(
      Map<String, String> pageValues)
      throws IOException {
    new File(projectPath + File.separatorChar + "conf").mkdirs();

    for (String env : new String[]{"Development", "Test",
        "Production"}) {
      createOverrideProperties(pageValues, env);
      createPlaceholderProperties(pageValues, env);
    }
  }

  void createPlaceholderProperties(
      Map<String, String> pageValues,
      String environment)
      throws FileNotFoundException, IOException {
    Properties props = new Properties();

    loadProperties(props, PropertiesType.Placeholder, environment);

    String beanId =
        "true".equals(pageValues.get("is.queue")) ? "queueJndiTemplate"
            : "topicJndiTemplate";

    addNewProperty(props, beanId + ".provider.url", pageValues
        .get("provider.url"));
    addNewProperty(props, beanId + ".principal", pageValues
        .get("security.principal"));
    addNewProperty(props, beanId + ".credentials", pageValues
        .get("security.credentials"));

    saveProperties(props, PropertiesType.Placeholder,
        environment);
  }

  void createOverrideProperties(
      Map<String, String> pageValues,
      String environment)
      throws IOException,
      FileNotFoundException {
    Properties props = new Properties();

    loadProperties(props, PropertiesType.Override, environment);

    addNewProperty(props, pageValues.get("actor.name")
        + ("true".equals(pageValues.get("is.queue")) ? "Queue.jndiName"
        : "Topic.jndiName"), pageValues.get("destination.name"));
    addNewProperty(props,
        ("true".equals(pageValues.get("is.queue")) ? "Queue" : "Topic")
            + "ConnectionFactory.jndiName", pageValues
        .get("connection.factory.jndi.name"));

    if (!"".equals(pageValues.get("user.name"))) {
      addNewProperty(
          props,
          "true".equals(pageValues.get("is.queue")) ?
              "CredentialsQueueConnectionFactory"
              : "CredentialsTopicConnectionFactory" + ".username", pageValues
          .get("user.name"));
      addNewProperty(
          props,
          "true".equals(pageValues.get("is.queue")) ?
              "CredentialsQueueConnectionFactory"
              : "CredentialsTopicConnectionFactory" + ".password", pageValues
          .get("password"));
    }

    saveProperties(props, PropertiesType.Override, environment);
  }

  void saveProperties(Properties props,
                      PropertiesType type, String environment)
      throws IOException,
      FileNotFoundException {
    String prefix = projectPath + File.separatorChar + "conf";

    new File(prefix).mkdirs();

    FileOutputStream fos = null;

    try {
      fos = new FileOutputStream(prefix + File.separatorChar
          + type.toString().toLowerCase() + ".properties." + environment);
      props.store(fos, environment + " " + type + " Property File");
    } finally {
      if (fos != null) {
        fos.close();
      }
    }
  }

  void loadProperties(Properties props,
                      PropertiesType type, String environment)
      throws IOException,
      FileNotFoundException {
    File propsFile = new File(projectPath + File.separatorChar + "conf"
        + File.separatorChar + type.toString().toLowerCase() + ".properties."
        + environment);

    if (propsFile.exists()) {
      FileInputStream fis = null;

      try {
        fis = new FileInputStream(propsFile);
        props.load(fis);
      } finally {
        if (fis != null) {
          fis.close();
        }
      }
    }
  }

  void addNewProperty(Properties props, String key, String value) {
    if (props != null && !props.containsKey(key)) {
      props.put(key, value);
    }
  }
}
