/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.MdsTest;
import midstream.core.util.WizardUtils;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.util.FileCopyUtils;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class MdsTestHandler extends BaseHandler {

  public MdsTestHandler(String projectPath) {
    super(projectPath);
  }

  public URLClassLoader createMdsTestClassLoader(
      String libDir,
      final boolean useClientJar) {
    List<URL> urls =
        new ProjectHandler(projectPath).getProjectClassUrls(libDir,
            useClientJar);

    return new URLClassLoader(urls.toArray(new URL[0]));
  }

  public String createMdsTest(
      Map<String, Object> pageValues) {
    Document document = DocumentHelper.createDocument();
    Element root = document.addElement("mds");
    root.addAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        .addAttribute("xsi:noNamespaceSchemaLocation", "mdstest.xsd");

    String testName = "";

    switch (MdsTest.TestType.valueOf(pageValues.get("test.type").toString())) {
      case REMOTE:
        testName = (String) pageValues.get("test.name");
        String method = (String) pageValues.get("method");
        method = method.contains("(") ? method.substring(0, method.indexOf('('))
            : method;

        Element test = root.addElement("param-test").addAttribute("name",
            testName)
            .addAttribute("service", (String) pageValues.get("service"))
            .addAttribute("method", method);
        for (String[] value : (String[][]) pageValues.get("parameter.values")) {
          test.addElement("param").addAttribute("type", value[0]).addCDATA(
              value[1].trim());
        }
        break;
      case HTTP_POST:
        testName = (String) pageValues.get("post.test.name");
        root.addElement("post-test")
            .addAttribute("name", testName)
            .addAttribute("url", pageValues.get("post.url").toString())
            .addElement("payload").addCDATA(
            (String) pageValues.get("post.payload"));
        break;
      case FREE_FORM:
        testName = (String) pageValues.get("free.form.test.name");
        root.addElement("freeform-test").addAttribute("name",
            testName)
            .addElement("script").addCDATA(
            (String) pageValues.get("free.form.script"));
        break;
    }

    WizardUtils.outputXmlDocument(new File(projectPath + File.separatorChar
        + "mds" + File.separatorChar + testName + ".xml"), document);
    return testName;
  }

  public void deleteMdsTest(
      final String testName) {
    WizardUtils.deleteFile(projectPath + File.separatorChar + "mds"
        + File.separatorChar + testName + ".xml");
  }

  public List<String> getMdsTestNames() {
    List<String> names = null;
    File mdsTestDir = new File(projectPath + File.separatorChar + "mds");

    if (mdsTestDir.exists()) {
      File[] tests = mdsTestDir.listFiles(new FilenameFilter() {
        public boolean accept(File file, String fileName) {
          return fileName.endsWith(".xml");
        }
      });
      if (tests != null && tests.length > 0) {
        names = new ArrayList<String>(tests.length);
        for (File file : tests) {
          names.add(file.getName().replace(".xml", ""));
        }
      }
    }

    return names;
  }

  public List<String> getServices()
      throws IOException {
    Document document = readApplicationContext();
    List ifaces = document
        .selectNodes(
            "//ns:bean[normalize-space(@class)='midstream.framework.remoting.httpinvoker.HttpInvokerServiceExporter']/ns:property[normalize-space(@name)='serviceInterface']/@value");
    List<String> services = new ArrayList<String>(ifaces.size());

    for (Object iface : ifaces) {
      services.add(((Attribute) iface).getText());
    }

    return services;
  }

  public List<String> getMethodNames(
      String serviceClass) {
    List<String> names = null;
    ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();

    try {
      Thread.currentThread().setContextClassLoader(
          createMdsTestClassLoader(projectPath + File.separatorChar
              + "lib", false));

      Method[] methods = Thread.currentThread().getContextClassLoader()
          .loadClass(serviceClass).getDeclaredMethods();

      names = new ArrayList<String>(methods.length);

      for (Method m : methods) {
        StringBuffer name = new StringBuffer(m.getName()).append("(");

        for (int i = 0; i < m.getParameterTypes().length; ++i) {
          name.append(m.getParameterTypes()[i].getSimpleName());

          if (i < (m.getParameterTypes().length - 1)) {
            name.append(", ");
          }
        }

        name.append(") ").append(m.getReturnType().getSimpleName());
        names.add(name.toString());
      }
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    } finally {
      Thread.currentThread().setContextClassLoader(oldClassLoader);
    }

    return names;
  }

  public List<String> getParameterTypes(
      final String serviceClass,
      final int methodIndex) {
    List<String> typeNames = null;
    ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();

    try {
      Thread.currentThread().setContextClassLoader(
          createMdsTestClassLoader(projectPath + File.separatorChar
              + "lib", false));

      Method[] methods = Thread.currentThread().getContextClassLoader()
          .loadClass(serviceClass).getDeclaredMethods();
      Method method = methods[methodIndex];
      Class[] paramTypes = method.getParameterTypes();

      if (paramTypes != null) {
        typeNames = new ArrayList<String>(paramTypes.length);
        for (Class type : paramTypes) {
          typeNames.add(type.getName());
        }
      }
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    } finally {
      Thread.currentThread().setContextClassLoader(oldClassLoader);
    }

    return typeNames;
  }

  public void convertTestToScript(String testPath,
                                  String scriptsLocation)
      throws IOException {
    Document document = WizardUtils.readXmlDocument(new File(testPath), true);

    if (document.selectSingleNode("/mds/param-test") != null) {
      String serviceName = document.selectSingleNode("//param-test/@service")
          .getText();

      if (serviceName.contains(".")) {
        serviceName = serviceName.substring(serviceName.lastIndexOf('.') + 1);
      }

      String methodName = document.selectSingleNode("//param-test/@method")
          .getText();
      List nodes = document.selectNodes("//param");

      int count = 1;
      for (Object node : nodes) {
        Element elem = (Element) node;
        WizardUtils.writeFile(scriptsLocation + File.separatorChar
            + serviceName + "_" + methodName + "_param" + count + ".script",
            elem.getTextTrim(), true);
        ++count;
      }
    } else if (document.selectSingleNode("/mds/post-test") != null) {
      String payload = document.selectSingleNode("//payload").getText().trim();
      String url =
          document.selectSingleNode("//post-test/@url").getText().trim();

      String contents = new String(FileCopyUtils.copyToByteArray(
          MdsTestHandler.class.getClassLoader().getResourceAsStream(
              "midstream/core/templates/components/mds_load_test_post_http.template")));
      contents = contents.replaceFirst("@URL@", url)
          .replaceFirst("@PAYLOAD@",
              payload.replaceAll("\n", "").replaceAll("\"", "\\\\\\\\\""));

      WizardUtils.writeFile(scriptsLocation + File.separatorChar
          + document.selectSingleNode("//post-test/@name").getText()
          + ".script", contents,
          true);
    } else {
      WizardUtils.writeFile(scriptsLocation + File.separatorChar
          + document.selectSingleNode("//freeform-test/@name").getText()
          + ".script", document.selectSingleNode("//script").getText().trim(),
          true);
    }
  }

  public MdsTest parseTest(String testName)
      throws IOException {
    MdsTest test = null;

    Document document = WizardUtils.readXmlDocument(new File(projectPath
        + File.separatorChar + "mds" + File.separatorChar + testName + ".xml"),
        true);

    if (document != null) {
      test = new MdsTest();

      if (document.selectSingleNode("/mds/param-test") != null) {
        test.setTestType(MdsTest.TestType.REMOTE);
        test.setTestName(document.selectSingleNode("//param-test/@name")
            .getText());
        test.setServiceName(document.selectSingleNode("//param-test/@service")
            .getText());
        test.setMethodName(document.selectSingleNode("//param-test/@method")
            .getText());
        List nodes = document.selectNodes("//param");
        List<String[]> params = new ArrayList<String[]>(nodes.size());

        for (Object node : nodes) {
          Element elem = (Element) node;
          params.add(new String[]{elem.attributeValue("type"),
              elem.getTextTrim()});
        }

        test.setParameters(params);
      } else if (document.selectSingleNode("/mds/post-test") != null) {
        test.setTestType(MdsTest.TestType.HTTP_POST);
        test.setTestName(document.selectSingleNode("//post-test/@name")
            .getText());
        test.setPostUrl(document.selectSingleNode("//post-test/@url")
            .getText());
        test.setPostPayload(
            document.selectSingleNode("//payload").getText().trim());
      } else {
        test.setTestType(MdsTest.TestType.FREE_FORM);
        test.setTestName(document.selectSingleNode("//freeform-test/@name")
            .getText());
        test.setScript(document.selectSingleNode("//script").getText().trim());
      }
    }

    return test;
  }
}
