/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.core.handler;

import midstream.core.util.WizardUtils;
import org.dom4j.Element;
import org.springframework.util.FileCopyUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Juan Hall
 */
public class ProjectDependencyHandler extends BaseHandler {

  public ProjectDependencyHandler(String projectPath) {
    super(projectPath);
  }

  private static List<String[]> buildLibraryList(List libs, String namespace) {
    List<String[]> libList = new ArrayList<String[]>(5);

    for (Object node : libs) {
      String[] info = new String[3];
      info[0] = ((Element) node).selectSingleNode("./" + namespace + "groupId")
          .getText();
      info[1] = ((Element) node).selectSingleNode(
          "./" + namespace + "artifactId").getText();
      info[2] = ((Element) node).selectSingleNode("./" + namespace + "version")
          .getText();
      libList.add(info);
    }

    Collections.sort(libList, new DependencyComparator());

    return libList;
  }

  public List<String> getAvailableLibraries(
      final String root) {
    List<String> libs = new ArrayList<String>(5);

    try {
      if (availableDependencyListThread.isAlive()) {
        try {
          availableDependencyListThread.join(5000);
        } catch (InterruptedException e) {
          // Ignore interruption
        }
      }

      File dependenciesFile = new File(System.getProperty("user.home") + File
          .separatorChar + ".midstream" + File.separatorChar +
          "available-dependencies.list");

      if (!dependenciesFile.exists()) {
        new DependencyFinder(projectPath).run();
      }

      String contents;

      if ("/".equals(root)) {
        contents =
            new String(FileCopyUtils.copyToByteArray(dependenciesFile));
      } else {
        URLConnection connection = new URL(new ProjectHandler(projectPath)
            .getBuildFileProps()
            .getProperty("remote.repository",
                "http://repo1.maven.org/maven2/") +
            root)
            .openConnection();
        contents = new String(
            FileCopyUtils.copyToByteArray(connection.getInputStream()));
      }

      Matcher m = Pattern.compile(
          ".*<img src=\"/icons/folder.gif\" alt=\"\\[DIR\\]\"> <a href=\"(.*)/\">.*")
          .matcher(contents);
      while (m.find()) {
        libs.add(m.group(1));
      }
    } catch (IOException e) {
      JOptionPane.showMessageDialog(null, e,
          "Error Retrieving Dependency Information",
          JOptionPane.ERROR_MESSAGE);
    }
    return libs;
  }

  public List<String[]> getProjectLibraryList()
      throws IOException {
    return buildLibraryList(WizardUtils.readPom(
        new File(projectPath + File.separatorChar + "pom.xml")).selectNodes(
        "//ns:dependency"), "ns:");
  }

  private static final Pattern VERSION_PATTERN = Pattern.compile(
      ".*<img src=\"/icons/folder.gif\" alt=\"\\[DIR\\]\"> <a href=\"(.*)/\">");

  public String getLatestDependencyVersion(
      final String groupId,
      final String artifactId) {
    try {
      URLConnection connection = new URL(new ProjectHandler(projectPath)
          .getBuildFileProps()
          .getProperty("remote.repository", "http://repo1.maven.org/maven2/") +
          "/" + groupId.replace('.', '/') + '/' + artifactId).openConnection();
      String contents = new String(
          FileCopyUtils.copyToByteArray(connection.getInputStream()));
      Matcher m = VERSION_PATTERN.matcher(contents);
      String latestVersion = "";
      String releaseCandidate = "";

      while (m.find()) {
        String version = m.group(1);

        // TODO: Remove Hack - Yeah, yeah...I know, I know. But <rant> if
        //       devs would just keep the maven-metadata.xml files up2date then
        //       I wouldn't have to resort to such atrocities</rant>.

        if (version.toLowerCase().contains("rc")) {
          if (version.compareTo(releaseCandidate) > 0) {
            releaseCandidate = version;
          }

          continue;
        }

        if (version.compareTo(latestVersion) > 0 &&
            Character.isDigit(version.charAt(0)) &&
            !isDateVersion(version)) {
          if ((version.startsWith(latestVersion) &&
              version.substring(latestVersion.length())
                  .matches("(\\d|\\.).*")) ||
              !version.startsWith(latestVersion)) {
            latestVersion = version;
          }
        } else if (latestVersion.startsWith(version) &&
            !latestVersion.substring(version.length()).matches("(\\d|\\.).*")) {
          latestVersion = version;
        }
      }

      if ("".equals(latestVersion)) {
        latestVersion = releaseCandidate;
      }

      return latestVersion;
    } catch (IOException e) {
      JOptionPane.showMessageDialog(null, e,
          "Error Retrieving Dependency Information",
          JOptionPane.ERROR_MESSAGE);
    }

    return null;
  }

  private static final SimpleDateFormat dateFormat =
      new SimpleDateFormat("yyyyMMdd");

  private static Date BEGIN_DATE;

  static {
    try {
      BEGIN_DATE = dateFormat.parse("19000101");
    } catch (ParseException e) {
      BEGIN_DATE = new Date();
    }
  }

  private static final Date NOW = new Date();

  private static boolean isDateVersion(String version) {
    if (version.length() >= 8) {
      try {
        Date date = dateFormat.parse(version.substring(0, 8));
        return date.after(BEGIN_DATE) && date.before(NOW);
      } catch (ParseException e) {
        return false;
      }
    }

    return false;
  }

  public String[] getLatestArtifactInfo(String artifactId,
                                        List<String[]> artifacts) {
    String[] found = null;

    for (String[] artifact : artifacts) {
      if (artifact[1].equals(artifactId)) {
        if (found == null || artifact[2].compareTo(found[2]) > 0) {
          found = artifact;
        }
      }
    }

    return found;
  }

  static Thread availableDependencyListThread;

  public void updateAvailableDependencyList() {
    availableDependencyListThread = new Thread(new DependencyFinder(null));
    availableDependencyListThread.start();
  }

  public void cleanUpAvailableDependencyList() {
    if (availableDependencyListThread != null) {
      if (availableDependencyListThread.isAlive()) {
        availableDependencyListThread.interrupt();
      }

      availableDependencyListThread = null;
    }
  }
}

/**
 * @author Juan Hall
 */
class DependencyComparator implements Comparator<String[]> {
  public int compare(String[] arg0, String[] arg1) {
    if (arg0[0].compareTo(arg1[0]) != 0) {
      return arg0[0].compareTo(arg1[0]);
    } else if (arg0[1].compareTo(arg1[1]) != 0) {
      return arg0[1].compareTo(arg1[1]);
    } else {
      return arg0[2].compareTo(arg1[2]);
    }
  }
}

/**
 * @author Juan Hall
 */
class DependencyFinder implements Runnable {
  private String projectPath;

  public DependencyFinder(final String projectPath) {
    this.projectPath = projectPath;
  }

  public void run() {
    Properties props;

    try {
      props = new ProjectHandler(projectPath).getBuildFileProps();
    } catch (Exception e) {
      props = new Properties();
    }

    File pluginDir = new File(
        System.getProperty("user.home") + File.separatorChar + ".midstream");

    pluginDir.mkdirs();

    try {
      URLConnection connection = new URL(props
          .getProperty("remote.repository", "http://repo1.maven.org/maven2/"))
          .openConnection();
      FileCopyUtils.copy(connection.getInputStream(), new FileOutputStream(
          pluginDir.getAbsolutePath() + File.separatorChar +
              "available-dependencies.list"));
    } catch (Throwable th) {
      JOptionPane.showMessageDialog(null, th,
          "Error Retrieving Dependency Information",
          JOptionPane.ERROR_MESSAGE);
    }
  }
}