/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.WizardUtils;
import org.springframework.util.FileCopyUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * @author Juan Hall
 */
public class ProjectHandler extends BaseHandler {

  public static final String[] DEFAULT_LIBRARIES =
      {"commons-logging:commons-logging",
          "com.jamonapi:jamon", "junit:junit", "log4j:log4j",
          "org.springframework:spring", "midstream-framework.jar",
          "javancss:javancss", "asm:asm", "javancss:ccl", "cobertura:cobertura",
          "oro:oro"};

  public static final String[] DEFAULT_WAR_LIBRARIES =
      {"com.jamonapi:jamon", "org.springframework:spring",
          "midstream-framework.jar", "log4j:log4j"};

  public ProjectHandler(String projectPath) {
    super(projectPath);
  }

  public void createProjectDirs() {
    String[] projectDirs = new String[]{"src",
        "test" + File.separatorChar + "src", "build", "dist", "mds", "conf",
        "lib"};

    for (String dir : projectDirs) {
      new File(projectPath + File.separatorChar + dir).mkdirs();
    }
  }

  public void createConfigFiles()
      throws IOException {
    final String[] files =
        new String[]{"log4j.xml", "web.xml", "applicationContext.xml",
            "context.xml"};
    for (String file : files) {
      WizardUtils.copyFile(ProjectHandler.class.getClassLoader()
          .getResourceAsStream(
          "midstream/core/templates/project/" + file),
          projectPath + File.separatorChar + "conf"
              + File.separatorChar + file, false);
    }
  }

  public void createBuildFiles(
      Map<String, String> pageValues,
      String pluginId) throws IOException {
    WizardUtils.copyFile(ProjectHandler.class.getClassLoader()
        .getResourceAsStream(
        "midstream/core/templates/project/build.properties"),
        projectPath + File.separatorChar + "build.properties", false);

    Properties props = getBuildFileProps();
    props.put("deployment.prefix", pageValues.get("deployment.prefix"));
    props.put("deployment.version", pageValues.get("deployment.version"));
    props.put("project.name", pageValues.get("project.name"));
    Properties overrideProps = getOverrideProperties();

    props.put("remote.repository", overrideProps.getProperty(
        "remote.repository", props.getProperty("remote.repository")));
    props.put("remote.backup.repository", overrideProps.getProperty(
        "remote.backup.repository",
        props.getProperty("remote.backup.repository")));
    props.put("deploy.dir", overrideProps.getProperty("deploy.dir",
        "${env.CATALINA_HOME}/webapps"));

    saveBuildFileProps(pluginId, props);

    String contents = WizardUtils.readStream(ProjectHandler.class
        .getClassLoader().getResourceAsStream(
        "midstream/core/templates/project/build.xml"));
    contents = contents.replaceAll("\\$PROJECT_NAME\\$", pageValues
        .get("project.name"));
    WizardUtils.writeFile(projectPath + File.separatorChar
        + "build.xml", contents, false);

    contents = WizardUtils.readStream(ProjectHandler.class.getClassLoader()
        .getResourceAsStream(
        "midstream/core/templates/project/pom.xml"));
    contents = contents.replaceAll("\\$PROJECT_NAME\\$", pageValues
        .get("project.name"));
    WizardUtils.writeFile(projectPath + File.separatorChar
        + "pom.xml", contents, false);

    WizardUtils.addLatestDependencies(projectPath,
        DEFAULT_WAR_LIBRARIES);
    WizardUtils.updateBuildFileWarLibraries(projectPath,
        DEFAULT_WAR_LIBRARIES, true);
  }

  void saveBuildFileProps(String pluginId,
                          Properties props)
      throws IOException, FileNotFoundException {
    props.store(new FileOutputStream(projectPath + File.separatorChar
        + "build.properties"), "Generated by MidStream (" + pluginId + ")");
  }

  public Properties getOverrideProperties() {
    Properties overrideProps = new Properties();

    try {
      overrideProps.load(new FileInputStream(System.getProperty("user.home")
          + File.separatorChar + ".midstream" + File.separatorChar
          + "override.properties"));
    } catch (Exception e) {
      // Ignore error loading overrides
    }

    return overrideProps;
  }

  public Properties getBuildFileProps()
      throws IOException {
    if (projectPath == null) {
      throw new IOException("The project path cannot be null");
    }

    File buildPropsFile = new File(projectPath + File.separatorChar
        + "build.properties");
    if (!buildPropsFile.exists()) {
      throw new FileNotFoundException("Cannot find 'build.properties'");
    }

    Properties props = new Properties();
    props.load(new FileInputStream(buildPropsFile));
    return props;
  }

  public void installBaseDependencies()
      throws IOException {
    FileCopyUtils
        .copy(ProjectHandler.class.getClassLoader().getResourceAsStream(
            "midstream/core/resources/lib/maven-artifact-ant-dep.jar"),
            new FileOutputStream(
                projectPath + File.separatorChar + "lib" + File.separatorChar +
                    "maven-artifact-ant-dep.jar"));

    FileCopyUtils
        .copy(ProjectHandler.class.getClassLoader().getResourceAsStream(
            "midstream/core/resources/lib/midstream-framework.jar"),
            new FileOutputStream(
                projectPath + File.separatorChar + "lib" + File.separatorChar +
                    "midstream-framework.jar"));
  }

  public List<URL> getProjectClassUrls(
      String libDir,
      final boolean useClientJar) {
    List<URL> urls = new ArrayList<URL>(5);

    try {
      if (useClientJar) {
        Properties props = getBuildFileProps();

        urls.add(new File(projectPath + File.separatorChar + "dist"
            + File.separatorChar + props.getProperty("deployment.prefix") + "-"
            + props.getProperty("deployment.version") + "-client.jar").toURL());
      } else {
        urls.add(new File(projectPath + File.separatorChar + "build/").toURL());
      }

      for (File file : new File(libDir).listFiles()) {
        urls.add(file.toURL());
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return urls;
  }

  public String getDeploymentHost() {
    String host = "localhost";

    Properties overrides =
        new ProjectHandler(projectPath).getOverrideProperties();

    if (overrides != null && overrides.containsKey("deployment.host")) {
      host = overrides.getProperty("deployment.host");
    }

    return host;
  }

  public String getDeploymentPort() {
    String port = "8080";

    Properties overrides =
        new ProjectHandler(projectPath).getOverrideProperties();

    if (overrides != null && overrides.containsKey("deployment.port")) {
      port = overrides.getProperty("deployment.port");
    }

    return port;
  }

  public String getHomeUrl() {
    String homeUrl;

    try {
      Properties props = getBuildFileProps();
      homeUrl = String.format(
          "http://%s:%s/%s-%s/remote/MonitorSummary",
          getDeploymentHost(), getDeploymentPort(),
          props.getProperty("deployment.prefix"), props
          .getProperty("deployment.version"));
    } catch (IOException ex) {
      homeUrl = null;
    }

    return homeUrl;
  }
}
