/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.SourceUtils;
import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * @author Juan Hall
 */
public class RemoteServiceHandler extends BaseHandler {

  private static final String[] JAVA_HTTP_LIBRARIES =
      {"javax.servlet:servlet-api"};

  private static final String[] XML_HTTP_LIBRARIES =
      {"javax.servlet:servlet-api",
          "org.jibx:jibx-bind", "org.jibx:jibx-run", "xpp3:xpp3", "bcel:bcel"};

  private static final String[] XML_RPC_LIBRARIES =
      {"org.apache.xmlrpc:xmlrpc-common",
          "org.apache.xmlrpc:xmlrpc-server", "ws-commons-util:ws-commons-util",
          "stax:stax-api", "woodstox:wstx-asl"};

  private static final String[] SOAP_LIBRARIES =
      {"org.codehaus.xfire:xfire-all", "stax:stax-api",
          "woodstox:wstx-asl", "javax.activation:activation", "jdom:jdom",
          "wsdl4j:wsdl4j"};

  public RemoteServiceHandler(String projectPath) {
    super(projectPath);
  }

  public String[] getLibraries(Map<String, String> pageValues) {
    if ("XML-HTTP".equals(pageValues.get("remoting.type"))) {
      return XML_HTTP_LIBRARIES;
    } else if ("XML-RPC".equals(pageValues.get("remoting.type"))) {
      return XML_RPC_LIBRARIES;
    } else if ("SOAP".equals(pageValues.get("remoting.type"))) {
      return SOAP_LIBRARIES;
    }

    return JAVA_HTTP_LIBRARIES;
  }

  public void updateBuildFile(
      Map<String, String> pageValues)
      throws IOException {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");

    if (!buildFile.exists()) {
      throw new FileNotFoundException("'build.xml' does not exist");
    }

    Document doc = WizardUtils.readXmlDocument(buildFile, false);
    Element root;

    if (doc == null) {
      throw new FileNotFoundException("Cannot find 'build.xml'");
    } else {
      root = doc.getRootElement();
    }

    createClientJarBuildConfiguration(pageValues, root);

    createJibxBuildConfiguration(pageValues, root);

    WizardUtils.outputXmlDocument(buildFile, doc);
  }

  private static void createJibxBuildConfiguration(
      Map<String, String> pageValues,
      Element root) {
    if (!"XML-HTTP".equals(pageValues.get("remoting.type"))) {
      return;
    }

    if (root.selectSingleNode("//path[@id='jibx.classpath']") == null) {
      Element path = root.addElement("path");
      path.addAttribute("id", "jibx.classpath");
      Element fileset = path.addElement("fileset").addAttribute("dir", "lib");
      fileset.addElement("include").addAttribute("name", "bcel-*.jar");
      fileset.addElement("include").addAttribute("name", "jibx-bind-*.jar");
      fileset.addElement("include").addAttribute("name", "jibx-run-*.jar");
      fileset.addElement("include").addAttribute("name", "xpp3-*.jar");
    }

    if (root.selectSingleNode("//target[@name='run-jibx']") == null) {
      Element target = root.addElement("target");
      target.addAttribute("name", "run-jibx")
          .addAttribute("depends", "compile");
      target.addElement("taskdef").addAttribute("name", "bind").addAttribute(
          "classname", "org.jibx.binding.ant.CompileTask").addAttribute(
          "classpathref", "jibx.classpath");
      Element bind = target.addElement("bind");
      bind.addAttribute("verbose", "true").addAttribute("binding",
          "conf/" + pageValues.get("service.name") + ".xml");
      Element cp = bind.addElement("classpath");
      cp.addElement("pathelement").addAttribute("path", "build");
      cp.addElement("path").addAttribute("refid", "jibx.classpath");

      Element war = (Element) root
          .selectSingleNode("//target[@name='dist']");
      war.addAttribute("depends", war.attributeValue("depends") + ", run-jibx");
    }
  }

  void createJibxBindingFile(
      Map<String, String> pageValues)
      throws IOException {
    String contents = WizardUtils
        .readStream(SourceUtils.class
            .getClassLoader()
            .getResourceAsStream(
            "midstream/core/templates/components/XmlHttpBinding.template"));

    contents = contents.replaceAll("\\$request\\$",
        pageValues.get("java.package") + "." + pageValues.get("request.name"))
        .replaceAll(
            "\\$response\\$",
            pageValues.get("java.package") + "."
                + pageValues.get("response.name"));

    WizardUtils.writeFile(projectPath + File.separatorChar + "conf"
        + File.separatorChar + pageValues.get("service.name") + ".xml",
        contents, false);
  }

  void createClientJarBuildConfiguration(
      Map<String, String> pageValues,
      Element root)
      throws IOException {
    if (root.selectSingleNode("//target[@name='dist-client-jar']") != null
        || !"HTTP".equals(pageValues.get("remoting.type"))) {
      return;
    }

    Properties buildProps = new ProjectHandler(projectPath).getBuildFileProps();
    StringBuffer clientJar = new StringBuffer("dist/").append(
        buildProps.getProperty("deployment.prefix")).append("-").append(
        buildProps.getProperty("deployment.version")).append("-client.jar");
    Element target = root.addElement("target");
    target.addAttribute("name", "dist-client-jar").addAttribute("depends",
        "compile");
    Element jar = target.addElement("jar");
    jar.addAttribute("destfile", clientJar.toString()).addElement("metainf")
        .addAttribute("dir", "conf").addElement("include").addAttribute("name",
        "clientContext.xml");
    jar.addElement("fileset").addAttribute("dir", "build")
        .addElement("include").addAttribute("name",
        "**/" + pageValues.get("service.name") + ".class");
  }

  public void createClientContextEntry(
      Map<String, String> pageValues)
      throws IOException {
    if (!"HTTP".equals(pageValues.get("remoting.type"))) {
      return;
    }

    new File(projectPath + File.separatorChar + "conf").mkdirs();

    File ctxFile = new File(projectPath + File.separatorChar + "conf"
        + File.separatorChar + "clientContext.xml");
    Document doc = WizardUtils.readXmlDocument(ctxFile, false);
    Element root;

    if (doc == null) {
      FileCopyUtils.copy(
          RemoteServiceHandler.class.getClassLoader().getResourceAsStream(
              "midstream/core/templates/project/applicationContext.xml"),
          new FileOutputStream(ctxFile));
      doc = WizardUtils.readXmlDocument(ctxFile, false);
    }

    root = doc.getRootElement();
    root.add(
        new Namespace("ns", "http://www.springframework.org/schema/beans"));

    if (doc
        .selectSingleNode("//ns:bean[@id='" + pageValues.get("java.package")
            + "." + pageValues.get("service.name") + "']") != null) {
      return;
    }

    Properties props = new ProjectHandler(projectPath).getBuildFileProps();
    StringBuffer url = new StringBuffer("http://localhost:8080/").append(
        props.getProperty("deployment.prefix")).append("-").append(
        props.getProperty("deployment.version")).append(
        "/remote/Remote").append(pageValues.get("service.name"));
    Element bean = root.addElement("bean").addAttribute("id",
        pageValues.get("java.package") + "." + pageValues.get("service.name"))
        .addAttribute("class",
            "midstream.framework.remoting.httpinvoker.HttpInvokerProxyFactoryBean");
    bean.addElement("property").addAttribute("name", "serviceUrl")
        .addAttribute("value", url.toString());
    bean.addElement("property").addAttribute("name", "serviceInterface")
        .addAttribute(
            "value",
            pageValues.get("java.package") + "."
                + pageValues.get("service.name"));

    WizardUtils.outputXmlDocument(ctxFile, doc);
  }

  public void createServiceContextEntry(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("java.package")
        + "." + pageValues.get("service.name") + "']") == null) {
      root.addElement("bean")
          .addAttribute(
              "id",
              pageValues.get("java.package") + "."
                  + pageValues.get("service.name")).addAttribute(
          "class",
          pageValues.get("java.package") + "."
              + pageValues.get("service.name") + "Impl");
    }

    if (doc.selectSingleNode("//ns:bean[@name='/Remote"
        + pageValues.get("service.name") + "']") == null) {
      String exporterName =
          "midstream.framework.remoting.httpinvoker.HttpInvokerServiceExporter";

      if ("XML-HTTP".equals(pageValues.get("remoting.type"))) {
        exporterName =
            "midstream.framework.remoting.xmlinvoker.XmlHttpInvokerServiceExporter";
      } else if ("XML-RPC".equals(pageValues.get("remoting.type"))) {
        exporterName =
            "midstream.framework.remoting.xmlinvoker.XmlRpcInvokerServiceExporter";
      } else if ("SOAP".equals(pageValues.get("remoting.type"))) {
        exporterName =
            "midstream.framework.remoting.xmlinvoker.SoapInvokerServiceExporter";
      }

      Element bean = root.addElement("bean").addAttribute("name",
          "/Remote" + pageValues.get("service.name")).addAttribute("class",
          exporterName);
      bean.addElement("property").addAttribute("name", "service")
          .addAttribute(
              "ref",
              pageValues.get("java.package") + "."
                  + pageValues.get("service.name"));
      bean.addElement("property").addAttribute("name", "serviceInterface")
          .addAttribute(
              "value",
              pageValues.get("java.package") + "."
                  + pageValues.get("service.name"));

      if ("XML-HTTP".equals(pageValues.get("remoting.type"))) {
        bean.addElement("property").addAttribute("name", "serviceMethod")
            .addAttribute("value", pageValues.get("service.method.name"));
        bean.addElement("property").addAttribute("name", "requestClass")
            .addAttribute(
                "value",
                pageValues.get("java.package") + "."
                    + pageValues.get("request.name"));
        bean.addElement("property").addAttribute("name", "responseClass")
            .addAttribute(
                "value",
                pageValues.get("java.package") + "."
                    + pageValues.get("response.name"));
      }
    }

    outputApplicationContext(doc);
  }

  public void createRemoteServiceClassTest(
      Map<String, String> pageValues)
      throws IOException {
    SourceUtils.createSourceClassTest(projectPath, pageValues
        .get("service.name")
        + "Impl", getTestStubs(pageValues), "XML-HTTP".equals(pageValues
        .get("remoting.type")) ? "XmlHttpRemoteServiceClass"
        : "RemoteServiceClass");
  }

  Map<String, String> getTestStubs(Map<String, String> pageValues) {
    Map<String, String> stubs = new HashMap<String, String>(5);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("service.name"));
    stubs.put("$method$", pageValues.get("service.method.name"));
    stubs.put("$request$", pageValues.get("request.name"));
    stubs.put("$response$", pageValues.get("response.name"));

    if ("XML-HTTP".equals(pageValues.get("remoting.type"))) {
      stubs.put("$methodUpper$", StringUtils.capitalize(pageValues
          .get("service.method.name")));
    }

    return stubs;
  }

  public String createRemoteServiceClass(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("service.name"));
    stubs.put("$method$", pageValues.get("service.method.name"));
    stubs.put("$request$", pageValues.get("request.name"));
    stubs.put("$response$", pageValues.get("response.name"));

    String templatePrefix = "XML-HTTP".equals(pageValues.get("remoting.type")) ?
        "XmlHttpRemoteServiceClass"
        : "RemoteServiceClass";

    if ("XML-HTTP".equals(pageValues.get("remoting.type"))) {
      createJibxBindingFile(pageValues);

      stubs.put("$typeName$", pageValues.get("request.name"));
      SourceUtils.createSourceClass(projectPath,
          pageValues.get("request.name"), stubs, "XmlHttpValueType");

      stubs.put("$typeName$", pageValues.get("response.name"));
      SourceUtils.createSourceClass(projectPath, pageValues
          .get("response.name"), stubs, "XmlHttpValueType");
    }

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("service.name")
        + "Impl", stubs, templatePrefix);
  }

  public void createRemoteServiceInterface(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("service.name"));
    stubs.put("$method$", pageValues.get("service.method.name"));
    stubs.put("$request$", pageValues.get("request.name"));
    stubs.put("$response$", pageValues.get("response.name"));

    String templatePrefix = "XML-HTTP".equals(pageValues.get("remoting.type")) ?
        "XmlHttpRemoteServiceInterface"
        : "RemoteServiceInterface";
    SourceUtils.createSourceClass(projectPath, pageValues.get("service.name"),
        stubs, templatePrefix);
  }
}
