/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.SourceUtils;
import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Namespace;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class SoapClientHandler extends BaseHandler {

  public static final String[] LIBRARIES =
      {"org.apache.axis:axis", "axis:axis-jaxrpc",
          "commons-discovery:commons-discovery", "javax.xml.soap:saaj-api",
          "wsdl4j:wsdl4j", "org.apache.axis:axis-ant", "cglib:cglib-nodep",
          "javax.activation:activation"};

  public SoapClientHandler(String projectPath) {
    super(projectPath);
  }

  public String createClientClass(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("client.name"));
    stubs.put("$mapping.package$", pageValues.get("mapping.package"));
    stubs.put("$service$", getNameAttributeFromWsdl(pageValues.get("wsdl.url"),
        "service"));
    stubs.put("$port.name$", getNameAttributeFromWsdl(pageValues
        .get("wsdl.url"), "port"));

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("client.name"), stubs, "SoapClientClass");
  }

  public void createClientClassTest(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("client.name"));
    stubs.put("$mapping.package$", pageValues.get("mapping.package"));
    stubs.put("$service$", getNameAttributeFromWsdl(pageValues.get("wsdl.url"),
        "service"));
    stubs.put("$port.name$", getNameAttributeFromWsdl(pageValues
        .get("wsdl.url"), "port"));
    stubs.put("$port.type.name$", getNameAttributeFromWsdl(pageValues
        .get("wsdl.url"), "portType"));

    SourceUtils.createSourceClassTest(projectPath, pageValues
        .get("client.name"), stubs, "SoapClientClass");
  }

  public void createContextEntry(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("client.name")
        + "Client']") == null) {
      Element bean = root.addElement("bean").addAttribute("id",
          pageValues.get("client.name") + "Client").addAttribute("class",
          "midstream.framework.remoting.client.AxisClientProxyFactoryBean");
      bean.addElement("property").addAttribute("name", "target")
          .addElement("bean").addAttribute(
          "class",
          pageValues.get("mapping.package") + "."
              + getNameAttributeFromWsdl(pageValues.get("wsdl.url"), "service")
              + "Locator");
      bean.addElement("property").addAttribute("name", "timeout").addAttribute(
          "value", pageValues.get("read.timeout"));
    }

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("java.package")
        + "." + pageValues.get("client.name") + "']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id",
          pageValues.get("java.package") + "." + pageValues.get("client.name"))
          .addAttribute(
              "class",
              pageValues.get("java.package") + "."
                  + pageValues.get("client.name"));
      bean.addElement("property").addAttribute("name", "serviceLocator")
          .addAttribute("ref", pageValues.get("client.name") + "Client");
    }

    outputApplicationContext(doc);
  }

  String getNameAttributeFromWsdl(String url, String attribute)
      throws IOException {
    File tempFile = File.createTempFile("wsdl", ".xml");
    WizardUtils.downloadFile(new URL(url), tempFile.getAbsolutePath());

    Document doc = WizardUtils.readXmlDocument(tempFile, false);

    if (!"wsdl".equals(doc.getRootElement().getNamespaceForURI(
        "http://schemas.xmlsoap.org/wsdl/"))) {
      doc.getRootElement().add(
          new Namespace("wsdl", "http://schemas.xmlsoap.org/wsdl/"));
    }

    if (doc.selectSingleNode("//wsdl:" + attribute + "/@name") == null) {
      throw new IOException("Unable to parse service name from: " + url);
    }

    return doc.selectSingleNode("//wsdl:" + attribute + "/@name").getText();
  }

  String getTargetNamespace(String url) throws IOException {
    File tempFile = File.createTempFile("wsdl", ".xml");
    WizardUtils.downloadFile(new URL(url), tempFile.getAbsolutePath());

    Document doc = WizardUtils.readXmlDocument(tempFile, false);

    if (!"wsdl".equals(doc.getRootElement().getNamespaceForURI(
        "http://schemas.xmlsoap.org/wsdl/"))) {
      doc.getRootElement().add(
          new Namespace("wsdl", "http://schemas.xmlsoap.org/wsdl/"));
    }

    if (doc.selectSingleNode("//wsdl:definitions/@targetNamespace") == null) {
      throw new IOException("Unable to parse target namespace from: " + url);
    }

    return doc.selectSingleNode("//wsdl:definitions/@targetNamespace")
        .getText();
  }

  public String[] updateBuildFile(
      Map<String, String> pageValues)
      throws IOException {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");

    if (!buildFile.exists()) {
      throw new FileNotFoundException("'build.xml' does not exist");
    }

    Document doc = WizardUtils.readXmlDocument(buildFile, false);
    Element root;

    if (doc == null) {
      throw new FileNotFoundException("Cannot find 'build.xml'");
    } else {
      root = doc.getRootElement();
    }

    if (root.selectSingleNode("//target[@name='generate-"
        + pageValues.get("client.name") + "-client-jar']") != null) {
      return null;
    }

    if (root.selectSingleNode("//path[@id='axis.classpath']") == null) {
      Element path = root.addElement("path").addAttribute("id",
          "axis.classpath");
      Element fileset = path.addElement("fileset").addAttribute("dir", "lib");
      fileset.addElement("include").addAttribute("name", "axis-*.jar");
      fileset.addElement("include").addAttribute("name", "axis-jaxrpc-*.jar");
      fileset.addElement("include").addAttribute("name", "commons-*.jar");
      fileset.addElement("include").addAttribute("name", "saaj-*.jar");
      fileset.addElement("include").addAttribute("name", "wsdl4j-*.jar");
    }

    String serviceName = getNameAttributeFromWsdl(pageValues.get("wsdl.url"),
        "service").toLowerCase();
    StringBuffer clientJar = new StringBuffer("lib/").append(serviceName)
        .append("-client.jar");
    Element target = root.addElement("target");
    target.addAttribute("name",
        "generate-" + pageValues.get("client.name") + "-client-jar")
        .addAttribute("depends", "init");
    target.addElement("delete").addAttribute("dir", "temp/bin");
    target.addElement("delete").addAttribute("dir", "temp/src");
    target.addElement("delete").addAttribute("file", clientJar.toString());
    target.addElement("mkdir").addAttribute("dir", "temp/bin");
    target.addElement("mkdir").addAttribute("dir", "temp/src");

    target.addElement("taskdef").addAttribute("resource",
        "axis-tasks.properties").addAttribute("classpathref", "axis.classpath");
    target.addElement("axis-wsdl2java").addAttribute("output", "temp/src")
        .addAttribute("url", pageValues.get("wsdl.url")).addElement("mapping")
        .addAttribute("namespace",
            getTargetNamespace(pageValues.get("wsdl.url"))).addAttribute(
        "package", pageValues.get("mapping.package"));

    target.addElement("javac").addAttribute("srcdir", "temp/src").addAttribute(
        "destdir", "temp/bin").addAttribute("debug", "true").addAttribute(
        "classpathref", "axis.classpath");
    Element jar = target.addElement("jar");
    jar.addAttribute("destfile", clientJar.toString()).addAttribute("basedir",
        "temp/bin");

    WizardUtils.outputXmlDocument(buildFile, doc);

    List<String> jars = new ArrayList<String>(Arrays.asList(LIBRARIES));
    jars.add(serviceName + "-client.jar");

    return jars.toArray(new String[0]);
  }
}
