/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.util;

import midstream.core.handler.ProjectDependencyHandler;
import midstream.core.handler.ProjectHandler;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.springframework.util.FileCopyUtils;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * @author Juan Hall
 */
public final class WizardUtils {

  private WizardUtils() {
  }

  public static String readStream(InputStream is) throws IOException {
    return FileCopyUtils.copyToString(new InputStreamReader(is));
  }

  public static void writeFile(String path, String contents, boolean overwrite)
      throws IOException {
    File file = new File(path);

    if (file.exists() && !overwrite) {
      return;
    }

    FileCopyUtils.copy(contents.getBytes(), file);
  }

  public static Document readXmlDocument(File xmlFile, boolean validate)
      throws IOException {
    if (!xmlFile.exists()) {
      return null;
    }

    FileInputStream fis = null;

    try {
      fis = new FileInputStream(xmlFile);
      return readXmlDocument(fis, validate);
    } finally {
      if (fis != null) {
        fis.close();
      }
    }
  }

  public static Document readXmlDocument(InputStream inputStream,
                                         boolean validate) throws IOException {
    Document doc;

    try {
      SAXReader reader = new SAXReader();
      if (!validate) {
        reader.setEntityResolver(new EntityResolver() {
          public InputSource resolveEntity(String publicId, String systemId) {
            return new InputSource(new StringReader(""));
          }
        });
      }

      doc = reader.read(inputStream);
    } catch (Exception e) {
      final IOException ioe = new IOException(e.getMessage());
      ioe.initCause(e);
      throw ioe;
    }

    return doc;
  }

  public static void outputXmlDocument(File outFile, Document doc) {
    final OutputFormat format = OutputFormat.createPrettyPrint();
    XMLWriter writer = null;

    try {
      writer = new XMLWriter(new FileWriter(outFile.getAbsolutePath()), format);
      writer.write(doc);
      writer.flush();
    } catch (IOException e) {
      e.printStackTrace();
    } finally {
      try {
        if (writer != null) {
          writer.close();
        }
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  public static void copyFile(InputStream is, String destination,
                              boolean overwrite) throws IOException {
    if (new File(destination).exists() && !overwrite) {
      return;
    }

    FileCopyUtils.copy(is, new FileOutputStream(destination));
  }

  public static void copyFile(String source, String destination,
                              boolean overwrite) throws IOException {
    File sourceFile = new File(source);

    if (sourceFile.exists()) {
      copyFile(new FileInputStream(sourceFile), destination, overwrite);
    }
  }

  public static boolean isBuildFileWarLibrary(String projectPath,
                                              String library)
      throws IOException {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");

    if (!buildFile.exists()) {
      throw new FileNotFoundException("'build.xml' does not exist");
    }

    Document doc = WizardUtils.readXmlDocument(buildFile, false);
    Element root;

    if (doc == null) {
      throw new FileNotFoundException("Cannot find 'build.xml'");
    } else {
      root = doc.getRootElement();
    }

    Element lib = (Element) root.selectSingleNode("//war/lib[@dir='lib']");

    return lib != null
        && lib.selectSingleNode("//./include[@name='"
        + (library.endsWith(".jar") ? library : getJarNameFromPom(
        projectPath, library)) + "']") != null;
  }

  public static void updateBuildFileWarLibraries(String projectPath,
                                                 String[] libraries,
                                                 boolean isAdd)
      throws IOException {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");

    if (!buildFile.exists()) {
      throw new FileNotFoundException("'build.xml' does not exist");
    }

    Document doc = WizardUtils.readXmlDocument(buildFile, false);
    Element root;

    if (doc == null) {
      throw new FileNotFoundException("Cannot find 'build.xml'");
    } else {
      root = doc.getRootElement();
    }

    Element lib = (Element) root.selectSingleNode("//war/lib[@dir='lib']");

    if (lib == null) {
      return;
    }

    Properties props = new ProjectHandler(projectPath).getOverrideProperties();
    String excludesProp = (String) props.get("deployment.excludes");
    List<String> excludes = new ArrayList<String>(5);

    if (excludesProp != null && !"".equals(excludesProp.trim())) {
      excludes.addAll(Arrays.asList(excludesProp.split("\\s")));
    } else {
      excludes.addAll(Arrays.asList(new String[]{"servlet-api",
          "commons-logging", "junit", "mockobjects-core"}));
    }

    boolean outputDoc = false;

    for (String name : libraries) {
      name = name.contains(":") ? name.split(":")[1] : name;
      Element jar = (Element) lib.selectSingleNode("//./include[@name='"
          + (name.endsWith(".jar") ? name
          : getJarNameFromPom(projectPath, name)) + "']");

      if (isAdd && jar == null) {
        if (excludes.contains(name.replace(".jar", ""))) {
          continue;
        }

        lib.addElement("include").addAttribute(
            "name",
            (name.endsWith(".jar") ? name
                : getJarNameFromPom(projectPath, name)));
        outputDoc = true;
      } else if (!isAdd && jar != null) {
        lib.remove(jar);
        outputDoc = true;
      }
    }

    if (outputDoc) {
      WizardUtils.outputXmlDocument(buildFile, doc);
    }
  }

  public static void downloadFile(final URL url, final String filePath)
      throws IOException {
    FileOutputStream fos = null;

    try {
      fos = new FileOutputStream(filePath);

      FileCopyUtils.copy(url.openConnection().getInputStream(), fos);
    } finally {
      if (fos != null) {
        fos.close();
      }
    }
  }

  public static void deleteFile(String fileName) {
    File file = new File(fileName);

    if (file.exists() && !file.isDirectory()) {
      file.delete();
    }
  }

  public static String getJarNameFromPom(final String projectPath,
                                         final String libName) {
    String jarName = null;

    try {
      Document doc = readPom(new File(projectPath + File.separatorChar
          + "pom.xml"));
      Element version = (Element) doc
          .selectSingleNode("//ns:dependency[ns:artifactId='" + libName
              + "']/ns:version");

      if (version != null) {
        return libName + "-" + version.getText() + ".jar";
      }
    } catch (IOException e) {
      e.printStackTrace();
    }

    return jarName;
  }

  public static void addLatestDependencies(final String projectPath,
                                           final String[] libInfos) {
    List<String[]> libs = new ArrayList<String[]>(5);

    for (String lib : libInfos) {
      if (lib.endsWith(".jar") || !lib.contains(":")) {
        continue;
      }

      String[] libInfo = lib.split(":");
      String version = new ProjectDependencyHandler(projectPath)
          .getLatestDependencyVersion(libInfo[0], libInfo[1]);

      if (version != null) {
        libs.add(new String[]{libInfo[0], libInfo[1], version});
      }
    }

    updateDependencies(projectPath, libs, true);
  }

  public static Document readPom(final File pomFile) throws IOException {
    Document doc = readXmlDocument(pomFile, false);
    doc.getRootElement().add(
        new Namespace("ns", "http://maven.apache.org/POM/4.0.0"));
    return doc;
  }

  public static void updateDependencies(final String projectPath,
                                        final List<String[]> libInfos,
                                        boolean isAdd) {
    try {
      final File pomFile = new File(projectPath + File.separatorChar
          + "pom.xml");
      Document doc = readPom(pomFile);

      boolean modified = false;

      for (String[] libInfo : libInfos) {
        Element dependencies = (Element) doc
            .selectSingleNode("//ns:dependencies");

        Element dependency = (Element) dependencies
            .selectSingleNode("//./ns:dependency[ns:groupId='" + libInfo[0]
                + "' and ns:artifactId='" + libInfo[1] + "' and ns:version='"
                + libInfo[2] + "']");

        if (isAdd) {
          if (dependency == null) {
            dependency = dependencies.addElement("dependency");
            dependency.addElement("groupId").addText(libInfo[0]);
            dependency.addElement("artifactId").addText(libInfo[1]);
            dependency.addElement("version").addText(libInfo[2]);
            modified = true;
          } else {
            Element scope = (Element) doc
                .selectSingleNode("//ns:dependency[ns:artifactId='"
                    + libInfo[1] + "']/ns:scope");
            if (scope != null) {
              dependency.remove(scope);
              modified = true;
            }
          }
        } else if (dependency != null) {
          dependencies.remove(dependency);

          deleteFile(projectPath + File.separatorChar + "lib"
              + File.separatorChar + libInfo[1] + "-" + libInfo[2] + ".jar");

          modified = true;
        }
      }

      if (modified) {
        outputXmlDocument(pomFile, doc);
      }
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public static Element addPlaceholderConfig(Document doc, Element root) {
    if (doc.selectSingleNode("//ns:bean[@id='placeholderConfig']") == null) {
      return root
          .addElement("bean")
          .addAttribute("id", "placeholderConfig")
          .addAttribute("class",
              "org.springframework.beans.factory.config.PropertyPlaceholderConfigurer");
    }

    return null;
  }
}
