/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.core.handler;

import junit.framework.TestCase;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.QName;
import org.junit.Test;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class AddBeanPropertyHandlerTest extends TestCase {

  @Test
  public void testAddBeanProperty() throws Exception {
    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.tab.index", 0);

    final Document doc = DocumentHelper.createDocument(
        DocumentHelper.createElement(new QName("beans", new Namespace("ns",
            "http://www.springframework.org/schema/beans"))));
    doc.getRootElement().addElement(new QName("bean", new Namespace("ns",
        "http://www.springframework.org/schema/beans")))
        .addAttribute("id", "testBean");

    AddBeanPropertyHandler addBeanPropertyHandler =
        new AddBeanPropertyHandler(null) {
          @Override
          protected Document readApplicationContext() {
            return doc;
          }

          @Override
          String createReference(Map<String, Object> pageValues, Element bean) {
            return "srcFile";
          }

          @Override
          protected void outputApplicationContext(Document doc) {
          }
        };
    assertEquals("srcFile",
        addBeanPropertyHandler.addBeanProperty(pageValues, "testBean"));
  }

  /*
   * TODO: Enhance test case to be more thorough
   */
  @Test
  public void testBuildClassNameList() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler("proj") {
      @Override
      protected String readFile(final String fileName) {
        assertEquals(
            "proj/conf/applicationContext.xml".replace('/', File.separatorChar),
            fileName);
        return "<beans><bean id=\"${CONF_DIR}\" class=\"\"/></beans>";
      }

      @Override
      protected void writeFile(final String fileName, final String contents,
                               final boolean overwrite) {
        assertEquals("proj/build/applicationContext.xml".replace('/',
            File.separatorChar), fileName);
        assertEquals("<beans><bean id=\"conf\" class=\"\"/></beans>", contents);
        assertTrue(overwrite);
      }
    };

    List<String> classNames = handler.buildClassNameList();
    assertTrue(classNames.contains("org.apache.commons.configuration.Configuration"));
  }

  @Test
  public void testGetMatchingItems() {
    assertEquals(Arrays.asList(new String[]{"aBA", "aba", "ba"}),
        Arrays.asList(new AddBeanPropertyHandler(null).getMatchingItems("ba",
            Arrays.asList(new String[]{"aBA", "ba", "d", "aba"}))));
  }

  @Test
  public void testGetContextBeanNames() {
    final Document doc = DocumentHelper.createDocument(
        DocumentHelper.createElement(new QName("beans", new Namespace("ns",
            "http://www.springframework.org/schema/beans"))));
    doc.getRootElement().addElement(new QName("bean", new Namespace("ns",
        "http://www.springframework.org/schema/beans")))
        .addAttribute("id", "testBean2");
    doc.getRootElement().addElement(new QName("bean", new Namespace("ns",
        "http://www.springframework.org/schema/beans")))
        .addAttribute("name", "testBean");

    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      protected Document readApplicationContext() {
        return doc;
      }
    };

    assertEquals(Arrays.asList(new String[]{"testBean", "testBean2"}),
        handler.getContextBeanNames());
  }

  @Test
  public void testCreateCollection() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      String createPropertiesProperty(Map<String, Object> pageValues,
                                      Element bean) {
        return "prop";
      }

      @Override
      String createMapProperty(Map<String, Object> pageValues, Element bean) {
        return "map";
      }

      @Override
      String createListProperty(Map<String, Object> pageValues, Element bean) {
        return "list";
      }

      @Override
      String createSetProperty(Map<String, Object> pageValues, Element bean) {
        return "set";
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("collection.type", "Properties");

    assertEquals("prop", handler.createCollection(pageValues, null));

    pageValues.put("collection.type", "Map");

    assertEquals("map", handler.createCollection(pageValues, null));

    pageValues.put("collection.type", "List");

    assertEquals("list", handler.createCollection(pageValues, null));

    pageValues.put("collection.type", "Set");

    assertEquals("set", handler.createCollection(pageValues, null));
  }

  @Test
  public void testCreatePropertiesProperty() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      protected BeanHandler getBeanHandler() {
        return new BeanHandler(null) {
          @Override
          void addPropertiesToPropertyElement(Element property,
                                              List<String[]> collection) {
            assertEquals(Arrays.asList(new String[]{"d", "e"}),
                Arrays.asList(collection.get(0)));
            property.addAttribute("d", "e");
          }
        };
      }

      @Override
      void addGetterSetter(String srcFile, String type, String name) {
        assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
            srcFile);
        assertEquals("java.util.Properties", type);
        assertEquals("myProp", name);
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.name", "myProp");
    List<String[]> collection = new ArrayList<String[]>(1);
    collection.add(new String[]{"d", "e"});
    pageValues.put("collection", collection);
    pageValues.put("add.getter.setter", Boolean.TRUE);

    Element bean =
        DocumentHelper.createElement("bean").addAttribute("class", "a.b.C");

    assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
        handler.createPropertiesProperty(pageValues, bean));
    assertNotNull(
        bean.selectSingleNode("./property[@name='myProp' and @d='e']"));
  }

  @Test
  public void testCreateMapProperty() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      protected BeanHandler getBeanHandler() {
        return new BeanHandler(null) {
          @Override
          void addMapToPropertyElement(Element property,
                                       List<String[]> collection) {
            assertEquals(Arrays.asList(new String[]{"d", "e"}),
                Arrays.asList(collection.get(0)));
            property.addAttribute("d", "e");
          }
        };
      }

      @Override
      void addGetterSetter(String srcFile, String type, String name) {
        assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
            srcFile);
        assertEquals("java.util.Map", type);
        assertEquals("myProp", name);
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.name", "myProp");
    List<String[]> collection = new ArrayList<String[]>(1);
    collection.add(new String[]{"d", "e"});
    pageValues.put("collection", collection);
    pageValues.put("add.getter.setter", Boolean.TRUE);

    Element bean =
        DocumentHelper.createElement("bean").addAttribute("class", "a.b.C");

    assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
        handler.createMapProperty(pageValues, bean));
    assertNotNull(
        bean.selectSingleNode("./property[@name='myProp' and @d='e']"));
  }

  @Test
  public void testCreateListProperty() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      protected BeanHandler getBeanHandler() {
        return new BeanHandler(null) {
          @Override
          void addListToPropertyElement(Element property,
                                        List<String[]> collection) {
            assertEquals(Arrays.asList(new String[]{"d", "e"}),
                Arrays.asList(collection.get(0)));
            property.addAttribute("d", "e");
          }
        };
      }

      @Override
      void addGetterSetter(String srcFile, String type, String name) {
        assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
            srcFile);
        assertEquals("java.util.List", type);
        assertEquals("myProp", name);
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.name", "myProp");
    List<String[]> collection = new ArrayList<String[]>(1);
    collection.add(new String[]{"d", "e"});
    pageValues.put("collection", collection);
    pageValues.put("add.getter.setter", Boolean.TRUE);

    Element bean =
        DocumentHelper.createElement("bean").addAttribute("class", "a.b.C");

    assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
        handler.createListProperty(pageValues, bean));
    assertNotNull(
        bean.selectSingleNode("./property[@name='myProp' and @d='e']"));
  }

  @Test
  public void testCreateSetProperty() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      protected BeanHandler getBeanHandler() {
        return new BeanHandler(null) {
          @Override
          void addSetToPropertyElement(Element property,
                                       List<String[]> collection) {
            assertEquals(Arrays.asList(new String[]{"d", "e"}),
                Arrays.asList(collection.get(0)));
            property.addAttribute("d", "e");
          }
        };
      }

      @Override
      void addGetterSetter(String srcFile, String type, String name) {
        assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
            srcFile);
        assertEquals("java.util.Set", type);
        assertEquals("myProp", name);
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.name", "myProp");
    List<String[]> collection = new ArrayList<String[]>(1);
    collection.add(new String[]{"d", "e"});
    pageValues.put("collection", collection);
    pageValues.put("add.getter.setter", Boolean.TRUE);

    Element bean =
        DocumentHelper.createElement("bean").addAttribute("class", "a.b.C");

    assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
        handler.createSetProperty(pageValues, bean));
    assertNotNull(
        bean.selectSingleNode("./property[@name='myProp' and @d='e']"));
  }

  @Test
  public void testCreateValue() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      void addGetterSetter(String srcFile, String type, String name) {
        assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
            srcFile);
        assertEquals("char", type);
        assertEquals("myProp", name);
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.name", "myProp");
    pageValues.put("value", "v");
    pageValues.put("value.type", "char");
    pageValues.put("add.getter.setter", Boolean.TRUE);

    Element bean =
        DocumentHelper.createElement("bean").addAttribute("class", "a.b.C");

    assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
        handler.createValue(pageValues, bean));
    assertNotNull(
        bean.selectSingleNode("./property[@name='myProp' and @value='v']"));
  }

  @Test
  public void testCreateReference() {
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(null) {
      @Override
      void addGetterSetter(String srcFile, String type, String name) {
        assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
            srcFile);
        assertEquals("a.b.D", type);
        assertEquals("myProp", name);
      }
    };

    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("property.name", "myProp");
    pageValues.put("reference.bean", "bean");
    pageValues.put("fqn.type", "a.b.D");
    pageValues.put("add.getter.setter", Boolean.TRUE);

    Element bean =
        DocumentHelper.createElement("bean").addAttribute("class", "a.b.C");

    assertEquals("src/a/b/C.java".replace('/', File.separatorChar),
        handler.createReference(pageValues, bean));
    assertNotNull(
        bean.selectSingleNode("./property[@name='myProp' and @ref='bean']"));
  }

  @Test
  public void testAddGetterSetter() throws Exception {
    final String projectPath = "a" + File.separatorChar + "b";
    AddBeanPropertyHandler handler = new AddBeanPropertyHandler(projectPath) {
      @Override
      protected String readFile(final String fileName) {
        assertEquals(projectPath + File.separatorChar + "S.java", fileName);
        return "class S {}";
      }

      @Override
      protected void writeFile(final String fileName, final String contents,
                               final boolean overwrite) {
        assertEquals(projectPath + File.separatorChar + "S.java", fileName);
        assertTrue(contents.contains("private String prop;"));
        assertTrue(contents.contains("public String getProp()"));
        assertTrue(
            contents.contains("public void setProp(final String prop)"));
        assertEquals(true, overwrite);
      }
    };

    handler.addGetterSetter("S.java", "String", "prop");
  }
}
