/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiManager;
import com.intellij.psi.PsiMethod;
import com.intellij.psi.PsiType;
import com.intellij.psi.search.GlobalSearchScope;
import com.thoughtworks.xstream.XStream;
import midstream.core.handler.AddBeanPropertyHandler;
import midstream.util.IdeUtils;
import org.springframework.util.StringUtils;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class CreateBeanDefinitionDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField selectedClassText;
  private JTextField beanNameText;
  private JTable propertiesTable;
  private boolean ok = false;
  private Map<String, Object> pageValues = new HashMap<String, Object>(5);
  private Project project;
  private String classFile;
  private String projectPath;

  public CreateBeanDefinitionDialog(JFrame parent, Project project,
                                    String projectPath, String classFile) {
    super(parent, "Create Bean Definition", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    this.project = project;
    this.projectPath = projectPath;
    this.classFile = classFile;

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    this.selectedClassText.setText(classFile);

    propertiesTable.setModel(new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return col == 2;
      }
    });

    populatePropertiesTable();

    IdeUtils.packAndCenter(this);
  }

  private void populatePropertiesTable() {
    PsiClass psiClass = PsiManager.getInstance(this.project)
        .findClass(classFile, GlobalSearchScope.projectScope(this.project));
    List<String> params = new ArrayList<String>(5);

    if (psiClass == null) {
      return;
    }

    for (PsiMethod method : psiClass.getMethods()) {
      if (PsiType.VOID.equals(method.getReturnType()) &&
          method.getName() != null &&
          method.getName().startsWith("set") &&
          method.getParameterList().getParameters().length == 1) {
        params
            .add(StringUtils.uncapitalize(method.getName().replace("set", "")));
      }
    }

    Object[][] dataVector = new Object[params.size()][];

    for (int i = 0; i < params.size(); ++i) {
      dataVector[i] = new Object[]{params.get(i), "", null};
    }
    ((DefaultTableModel) propertiesTable.getModel())
        .setDataVector(dataVector, new Object[]{"Name", "Value", ""});

    final JButton tableButton = new JButton("...");
    tableButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        int row = propertiesTable.getSelectedRow();
        int col = propertiesTable.getSelectedColumn();
        AddBeanPropertyDialog dialog =
            new AddBeanPropertyDialog((JFrame) getParent(), projectPath,
                null);
        dialog.setTitle("Set Bean Property");
        dialog.setVisible(true);

        if (dialog.isOk()) {
          if (AddBeanPropertyHandler.COLLECTION == dialog
              .getPageValues().get("property.tab.index")) {
            propertiesTable.setValueAt(
                dialog.getPageValues().get("collection.type")
                    + "="
                    + new XStream().toXML(dialog.getPageValues().get(
                    "collection")), row, 1);
          } else if (AddBeanPropertyHandler.REFERENCE == dialog
              .getPageValues().get("property.tab.index")) {
            propertiesTable.setValueAt(
                "ref=" + dialog.getPageValues().get("fqn.type"), row, 1);
          } else if (AddBeanPropertyHandler.VALUE == dialog
              .getPageValues().get("property.tab.index")) {
            propertiesTable
                .setValueAt(dialog.getPageValues().get("value"), row, 1);
          }
          propertiesTable.getCellEditor(row, col).stopCellEditing();
        }
      }
    });
    propertiesTable.getColumnModel().getColumn(2)
        .setCellEditor(new DefaultCellEditor(new JCheckBox()) {
          public Component getTableCellEditorComponent(JTable jTable,
                                                       Object object,
                                                       boolean b, int i,
                                                       int i1) {
            return tableButton;
          }
        });
    propertiesTable.getColumnModel().getColumn(2)
        .setCellRenderer(new TableCellRenderer() {
          public Component getTableCellRendererComponent(JTable jTable,
                                                         Object object,
                                                         boolean b,
                                                         boolean b1, int i,
                                                         int i1) {
            return tableButton;
          }
        });
  }

  public boolean isOk() {
    return ok;
  }

  private void onOK() {
    ok = true;

    if (validateInput()) {
      setPageValues();
      dispose();
    }
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  private boolean validateInput() {
    if ("".equals(this.beanNameText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A bean name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return false;
    }

    return true;
  }

  private void setPageValues() {
    this.pageValues.put("selected.class", this.classFile);
    this.pageValues.put("bean.name", this.beanNameText.getText().trim());
    Map<String, String> properties = new HashMap<String, String>(5);

    for (int i = 0; i < this.propertiesTable.getRowCount(); ++i) {
      properties.put((String) this.propertiesTable.getValueAt(i, 0),
          (String) this.propertiesTable.getValueAt(i, 1));
    }

    this.pageValues.put("properties", properties);
  }

  public Map<String, Object> getPageValues() {
    return this.pageValues;
  }
}
