/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.execution.filters.TextConsoleBuidlerFactory;
import com.intellij.execution.ui.ConsoleView;
import com.intellij.openapi.project.Project;
import midstream.core.ConsoleOperation;
import midstream.core.MdsTestRunner;
import midstream.core.handler.MdsTestHandler;
import midstream.core.handler.ProjectHandler;
import midstream.util.IdeUtils;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class MdsTestRunnerForm {
  private JPanel rootComponent;
  private JComboBox testCombo;
  private JTextField portText;
  private JTextField hostText;
  private JPanel toolbarPanel;
  private String projectPath;
  private Project project;
  private ConsoleView console;
  private JCheckBox useClientJar;

  public MdsTestRunnerForm(Project project) {
    this.project = project;
    projectPath = project.getProjectFilePath().substring(0, project
        .getProjectFilePath().indexOf(project.getProjectFile().getName()) - 1);

    final Insets margins = new Insets(0, 0, 0, 0);
    JButton runButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/run.gif")));
    runButton.setMargin(margins);
    runButton.setToolTipText("Run MDS Test");
    runButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        runMdsTest();
      }
    });
    JButton newButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/new.gif")));
    newButton.setMargin(margins);
    newButton.setToolTipText("New MDS Test");
    newButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        createNewMdsTest();
      }
    });
    JButton modifyButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/modify.gif")));
    modifyButton.setMargin(margins);
    modifyButton.setToolTipText("Modify MDS Test");
    modifyButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        modifyMdsTest();
      }
    });
    JButton deleteButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/delete.gif")));
    deleteButton.setMargin(margins);
    deleteButton.setToolTipText("Delete MDS Test");
    deleteButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        deleteMdsTest();
      }
    });
    JButton refreshButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/refresh.gif")));
    refreshButton.setMargin(margins);
    refreshButton.setToolTipText("Refresh");
    refreshButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        refreshMdsTests();
      }
    });

    JToolBar toolbar = new JToolBar(JToolBar.HORIZONTAL);
    toolbar.setBorderPainted(false);
    toolbar.setFloatable(false);
    toolbar.add(runButton);
    toolbar.add(newButton);
    toolbar.add(modifyButton);
    toolbar.add(deleteButton);
    toolbar.add(refreshButton);
    toolbarPanel.setLayout(new FlowLayout());
    toolbarPanel.add(toolbar);

    this.hostText.setText(new ProjectHandler(projectPath).getDeploymentHost());
    this.portText.setText(new ProjectHandler(projectPath).getDeploymentPort());

    refreshMdsTests();
  }

  private void modifyMdsTest() {
    MdsTestDialog dialog = new MdsTestDialog(
        IdeUtils.suggestParentWindow(project), projectPath,
        this.testCombo.getSelectedItem().toString());
    dialog.toFront();
    dialog.setVisible(true);

    if (dialog.isOk()) {
      new MdsTestHandler(projectPath).createMdsTest(getPageValues(dialog));
    }
  }

  private void deleteMdsTest() {
    new MdsTestHandler(projectPath)
        .deleteMdsTest(
            this.testCombo.getSelectedItem().toString());
    refreshMdsTests();
  }

  private Map<String, Object> getPageValues(MdsTestDialog dialog) {
    Map<String, Object> pageValues = new HashMap<String, Object>(5);
    pageValues.put("test.type", dialog.getTestType().toString());

    switch (dialog.getTestType()) {
      case REMOTE:
        pageValues.put("test.name", dialog.getTestName());
        pageValues.put("service", dialog.getService());
        pageValues.put("method", dialog.getMethod());
        pageValues.put("parameter.values", dialog.getParameterValues());
        break;
      case HTTP_POST:
        pageValues.put("post.test.name", dialog.getPostTestName());
        pageValues.put("post.url", dialog.getPostUrl());
        pageValues.put("post.payload", dialog.getPostPayload());
        break;
      case FREE_FORM:
        pageValues.put("free.form.test.name", dialog.getFreeFormTestName());
        pageValues.put("free.form.script", dialog.getFreeFormScript());
        break;
    }

    return pageValues;
  }

  private void runMdsTest() {
    if (this.testCombo.getSelectedItem() == null) {
      return;
    }

    Runnable runner = new Runnable() {
      public void run() {
        if (console == null) {
          console =
              TextConsoleBuidlerFactory.getInstance().createBuilder(project)
                  .getConsole();
        }

        final JComponent rootComponent = getRootComponent();
        final Map<String, String> formParams = new HashMap<String, String>(5);
        formParams.put("project.path", projectPath);
        formParams
            .put("mds.tests.path", projectPath + File.separatorChar + "mds");
        formParams.put("test.name", testCombo.getSelectedItem().toString());
        formParams.put("host", hostText.getText());
        formParams.put("port", portText.getText());
        formParams.put("use.client.jar",
            Boolean.toString(useClientJar.isSelected()));

        ConsoleOperation operation =
            new ConsoleOperationImpl(console, rootComponent);
        IdeUtils.createConsoleWindow(project, console.getComponent(),
            "MDS Test Runner Console")
            .show(new MdsTestRunner(operation, formParams));
      }
    };

    IdeUtils.rebuildProject(project, runner);
  }

  private void createNewMdsTest() {
    Runnable runner = new Runnable() {
      public void run() {
        MdsTestDialog dialog = new MdsTestDialog(
            IdeUtils.suggestParentWindow(project), projectPath);
        dialog.toFront();
        dialog.setVisible(true);

        if (dialog.isOk()) {
          Map<String, Object> pageValues = getPageValues(dialog);
          String testName =
              new MdsTestHandler(projectPath).createMdsTest(pageValues);

          refreshMdsTests();

          int index = 0;
          for (int i = 0; i < testCombo.getItemCount(); ++i) {
            if (testName.equals(testCombo.getItemAt(i))) {
              index = i;
              break;
            }
          }
          testCombo.setSelectedIndex(index);
        }
      }
    };

    IdeUtils.rebuildProject(project, runner);
  }

  private void refreshMdsTests() {
    List<String> names = new MdsTestHandler(projectPath).getMdsTestNames();

    if (names != null) {
      DefaultComboBoxModel model =
          new DefaultComboBoxModel(names.toArray(new String[0]));
      testCombo.setModel(model);
    }
  }

  public JComponent getRootComponent() {
    return rootComponent;
  }
}
