/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.ProjectHandler;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Document;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public class MonitoringForm {
  private JEditorPane htmlPane;
  private JButton refreshButton;
  private JButton backButton;
  private JButton homeButton;
  private JButton forwardButton;
  private JPanel toolbarPanel;
  private JTextField locationText;
  private String homeUrl;
  private String backUrl;
  private String forwardUrl;

  public MonitoringForm(Project project) {
    setHomeUrl(project);

    createHtmlPane();

    locationText.setText(homeUrl);

    createRefreshButton();
    createBackButton();
    createHomeButton();
    createForwardButton();
    createToolBar();
  }

  private void setHomeUrl(Project project) {
    String projectPath = project.getProjectFilePath().substring(0, project
        .getProjectFilePath().indexOf(project.getProjectFile().getName()) - 1);

    homeUrl = new ProjectHandler(projectPath).getHomeUrl();

    if (homeUrl == null) {
      Logger.getInstance(this.getClass().getName())
          .error("Unable to set home URL.");
      homeUrl = "error";
    }

    backUrl = homeUrl;
    forwardUrl = homeUrl;
  }

  private void createHtmlPane() {
    htmlPane.addHyperlinkListener(new HyperlinkListener() {
      public void hyperlinkUpdate(HyperlinkEvent hyperlinkEvent) {
        if (hyperlinkEvent.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
        {
          try {
            backUrl = htmlPane.getPage().toString();
            forwardUrl = hyperlinkEvent.getURL().toString();
            htmlPane.setPage(hyperlinkEvent.getURL());
          } catch (IOException e) {
            htmlPane.setText(e.getClass() + " : " + e.getMessage());
          }
        }
      }
    });
    htmlPane.setEditable(false);
    htmlPane.setContentType("text/html");

    try {
      htmlPane.setPage(homeUrl);
    } catch (IOException e) {
      // Ignore exception setting page to home URL
    }
  }

  private void createRefreshButton() {
    refreshButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/refresh.gif")));
    refreshButton.setMargin(new Insets(0, 0, 0, 0));
    refreshButton.setToolTipText("Refresh");
    refreshButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        try {
          if (htmlPane.getPage() == null) {
            return;
          }

          String page = htmlPane.getPage().toString();
          htmlPane.getDocument()
              .putProperty(Document.StreamDescriptionProperty, null);
          htmlPane.setPage(page);
        } catch (IOException e) {
          htmlPane.setText(e.getClass() + " : " + e.getMessage());
        }
      }
    });
  }

  private void createBackButton() {
    backButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/back.gif")));
    backButton.setMargin(new Insets(0, 0, 0, 0));
    backButton.setToolTipText("Back");
    backButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        try {
          forwardUrl = htmlPane.getPage().toString();
          htmlPane.setPage(backUrl);
        } catch (IOException e) {
          htmlPane.setText(e.getClass() + " : " + e.getMessage());
        }
      }
    });
  }

  private void createHomeButton() {
    homeButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/home.gif")));
    homeButton.setMargin(new Insets(0, 0, 0, 0));
    homeButton.setToolTipText("Home");
    homeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        try {
          htmlPane.getDocument()
              .putProperty(Document.StreamDescriptionProperty, null);
          htmlPane.setPage(homeUrl);
        } catch (IOException e) {
          htmlPane.setText(e.getClass() + " : " + e.getMessage());
        }
      }
    });
  }

  private void createForwardButton() {
    forwardButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/forward.gif")));
    forwardButton.setMargin(new Insets(0, 0, 0, 0));
    forwardButton.setToolTipText("Forward");
    forwardButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        try {
          backUrl = htmlPane.getPage().toString();
          htmlPane.setPage(forwardUrl);
        } catch (IOException e) {
          htmlPane.setText(e.getClass() + " : " + e.getMessage());
        }
      }
    });
  }

  private void createToolBar() {
    JToolBar toolbar = new JToolBar(JToolBar.HORIZONTAL);
    toolbar.setBorderPainted(false);
    toolbar.setFloatable(false);
    toolbar.add(refreshButton);
    toolbar.add(backButton);
    toolbar.add(homeButton);
    toolbar.add(forwardButton);
    toolbarPanel.setLayout(new FlowLayout());
    toolbarPanel.add(toolbar);
  }

  public JEditorPane getHtmlPane() {
    return htmlPane;
  }

  public void setHtmlPane(JEditorPane htmlPane) {
    this.htmlPane = htmlPane;
  }

  public JPanel getRootComponent() {
    return rootComponent;
  }

  public void setRootComponent(JPanel rootComponent) {
    this.rootComponent = rootComponent;
  }

  private JPanel rootComponent;
}
