/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.projectRoots.ProjectJdk;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowId;
import com.intellij.openapi.wm.ToolWindowManager;
import midstream.core.handler.ProjectHandler;
import midstream.util.IdeUtils;
import midstream.util.ProjectActionUtils;
import org.springframework.util.FileCopyUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class NewProjectAction extends AnAction {

  public void actionPerformed(AnActionEvent e) {
    validateSettings();
    IdeUtils.setSystemProperties();

    Application application = ApplicationManager.getApplication();
    NewProjectApplicationComponent component =
        application.getComponent(NewProjectApplicationComponent.class);
    component.launchDialog();

    if (component.isOk()) {
      try {
        Project project =
            ProjectActionUtils.loadProject(createProject(component));

        ToolWindow toolwindow =
            ToolWindowManager.getInstance(project)
                .getToolWindow(ToolWindowId.PROJECT_VIEW);
        if (toolwindow != null) {
          toolwindow.activate(null);
        }
      } catch (final Exception ex) {
        JOptionPane.showMessageDialog(null, ex, "Error Creating Project",
            JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  private void validateSettings() {
    File settingsFile =
        new File(System.getProperty("user.home") + File.separatorChar +
            ".m2" + File.separatorChar + "settings.xml");
    if (!settingsFile.exists()) {
      try {
        FileCopyUtils.copy(this.getClass().getClassLoader().getResourceAsStream(
            "midstream/core/templates/project/settings.xml"),
            new FileOutputStream(settingsFile));
      } catch (IOException e) {
        Logger.getInstance(this.getClass().getName()).error(e);
      }

      JOptionPane.showMessageDialog(null,
          String.format(
              "Note: If you are working in a proxied environment, please fill in the necessary proxy settings in:\n\n%s\n\nso that project dependencies will be resolved correctly.",
              settingsFile.getAbsolutePath()),
          "Settings Validation",
          JOptionPane.INFORMATION_MESSAGE);
    }
  }

  private Project createProject(
      final NewProjectApplicationComponent component)
      throws IOException {
    final String projectPath = component.getProjectLocation() + File
        .separatorChar + component.getProjectName();

    ProjectHandler projectHandler = new ProjectHandler(projectPath);
    projectHandler.createProjectDirs();

    Map<String, String> pageValues = new HashMap<String, String>(3);
    pageValues.put("deployment.prefix", component.getDeploymentPrefix());
    pageValues.put("deployment.version", component.getDeploymentVersion());
    pageValues.put("project.name", component.getProjectName());

    projectHandler
        .createBuildFiles(pageValues,
            IdeUtils.getPluginId());
    projectHandler.installBaseDependencies();
    projectHandler.createConfigFiles();

    final Project[] projects = new Project[1];

    try {
      ApplicationManager.getApplication().runWriteAction(new Runnable() {
        public void run() {
          try {
            projects[0] = ProjectActionUtils
                .createProject(projectPath, component);

            ProjectJdk jdk =
                ProjectActionUtils.setProjectJdk(component, projects[0]);

            ProjectActionUtils
                .createModule(projects[0], jdk, projectPath,
                    component.getProjectName());

            projects[0].save();

            IdeUtils
                .createLibraries(projects[0], projectPath,
                    ProjectHandler.DEFAULT_LIBRARIES);
          } catch (Exception ex) {
            throw new RuntimeException(ex);
          }
        }
      });
    } catch (Exception ex) {
      Logger.getInstance(this.getClass().getName()).error(ex);
      final IOException ioe =
          new IOException("Unable to create project location.");
      ioe.initCause(ex);
      throw ioe;
    }

    return projects[0];
  }
}