/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.projectRoots.ProjectJdk;
import midstream.util.IdeUtils;
import midstream.util.ProjectJdkDialogUtils;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

/**
 * @author Juan Hall
 */
public class NewProjectDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField projectNameField;
  private JTextField projectLocationField;
  private JButton browseButton;
  private JTextField deploymentPrefixField;
  private JTextField deploymentVersionField;
  private boolean ok = false;
  private JLabel graphicLabel;
  private JButton addButton;
  private JComboBox projectJdkCombo;
  private String projectJdkPath;
  private String projectJdkName;

  public NewProjectDialog() {
    super((JFrame) null, "New MidStream Project", true);
    setAlwaysOnTop(true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    final NewProjectDialog thisDialog = this;
    browseButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fileChooser.showOpenDialog(thisDialog);
        thisDialog.projectLocationField
            .setText(fileChooser.getSelectedFile().getAbsolutePath());
      }
    });
    addButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        NewProjectJdkDialog dialog = new NewProjectJdkDialog(thisDialog);
        dialog.setData(thisDialog);
        dialog.setVisible(true);
        if (dialog.isOk()) {
          dialog.getData(thisDialog);
          setProjectJdk(projectJdkCombo,
              getProjectJdkName(), getProjectJdkPath());
        }
      }
    });

    this.graphicLabel.setIcon(new ImageIcon(getClass().getClassLoader()
        .getResource(
        "midstream/core/resources/icons/projectIcon.gif")));

    DefaultComboBoxModel model =
        new DefaultComboBoxModel(ProjectJdkDialogUtils.getAvailableJdkNames());
    projectJdkCombo.setModel(model);

    projectNameField.addKeyListener(new KeyAdapter() {
      @Override
      public void keyReleased(KeyEvent keyEvent) {
        super.keyReleased(keyEvent);
        deploymentPrefixField
            .setText(projectNameField.getText().trim().toLowerCase());
      }
    });

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.projectNameField.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A project name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.projectLocationField.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A project location must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if (new File(this.projectLocationField.getText().trim() + File
        .separatorChar + this.projectNameField.getText().trim()).exists()) {
      JOptionPane.showMessageDialog(this, "The given location already exists.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.deploymentPrefixField.getText().trim())) {
      JOptionPane.showMessageDialog(this,
          "A deployment prefix must be entered.", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.deploymentVersionField.getText().trim())) {
      JOptionPane.showMessageDialog(this,
          "A deployment version must be entered.", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    if (this.projectJdkCombo.getSelectedItem() == null) {
      JOptionPane.showMessageDialog(this, "A project JDK must be selected.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }

    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public void setData(NewProjectApplicationComponent data) {
    projectNameField.setText(data.getProjectName());
    projectLocationField.setText(data.getProjectLocation());
    deploymentPrefixField.setText(data.getDeploymentPrefix());
    deploymentVersionField.setText(data.getDeploymentVersion());
    for (int i = 0; i < projectJdkCombo.getItemCount(); ++i) {
      if (projectJdkCombo.getItemAt(i).toString().equals(data.getProjectJdk()))
      {
        projectJdkCombo.setSelectedIndex(i);
        break;
      }
    }
  }

  public void getData(NewProjectApplicationComponent data) {
    data.setProjectName(projectNameField.getText());
    data.setProjectLocation(projectLocationField.getText().replaceFirst(
        "[\\\\/]" + this.projectNameField.getText().trim() + "$", ""));
    data.setDeploymentPrefix(deploymentPrefixField.getText());
    data.setDeploymentVersion(deploymentVersionField.getText());

    if (projectJdkCombo.getSelectedItem() != null) {
      data.setProjectJdk(projectJdkCombo.getSelectedItem().toString());
    }
  }

  public boolean isOk() {
    return ok;
  }

  public void setProjectJdk(JComboBox projectJdkCombo, String jdkName,
                            String jdkPath) {
    int index = -1;

    for (int i = 0; i < projectJdkCombo.getItemCount(); ++i) {
      if (projectJdkCombo.getItemAt(i).toString().equals(jdkName)) {
        index = i;
        break;
      }
    }

    if (index == -1) {
      index = projectJdkCombo.getItemCount();

      String jrePath = jdkPath + File.separatorChar + "jre";

      try {
        final ProjectJdk jdk =
            ProjectJdkDialogUtils.createJdk(jdkName, jrePath);

        ((DefaultComboBoxModel) projectJdkCombo.getModel())
            .addElement(jdk.getName());

        for (Field field : jdk.getClass().getDeclaredFields()) {
          field.setAccessible(true);
          if (!Modifier.isFinal(field.getModifiers()) &&
              jrePath.replace('\\', '/').equals(field.get(jdk))) {
            field.set(jdk, jdkPath.replaceAll("\\\\", "/"));
          }
        }

        ApplicationManager.getApplication().runWriteAction(new Runnable() {
          public void run() {
            ProjectJdkDialogUtils.addJdk(jdk);
          }
        });
      } catch (Exception ex) {
        Logger.getInstance(ProjectJdkDialogUtils.class.getName()).error(ex);
      }
    }

    projectJdkCombo.setSelectedIndex(index);
  }

  public String getProjectJdkPath() {
    return projectJdkPath;
  }

  public void setProjectJdkPath(final String projectJdkPath) {
    this.projectJdkPath = projectJdkPath;
  }

  public String getProjectJdkName() {
    return projectJdkName;
  }

  public void setProjectJdkName(final String projectJdkName) {
    this.projectJdkName = projectJdkName;
  }
}
