/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.core.handler.ProjectDependencyHandler;
import midstream.util.IdeUtils;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Juan Hall
 */
public class ProjectDependenciesDialog extends JDialog {
  private boolean ok;
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JCheckBox addToBuildPathCheckBox;
  private JCheckBox addToDeploymentUnitCheckBox;
  private JTabbedPane tabPane;
  private JCheckBox removeFromDeploymentUnitCheckBox;
  private JCheckBox removeFromBuildPathCheckBox;
  private JTable removeLibraryTable;
  private JTable updateLibraryTable;
  private JTree availableLibraryTree;
  private List<String> addLibraryList = new ArrayList<String>(5);

  enum TableType {
    Add, Update, Remove }

  public ProjectDependenciesDialog(JFrame parent, final String projectPath
  ) {
    super(parent, "Manage Project Dependencies", true);

    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    this.tabPane.setTitleAt(0, "Add Library");
    this.tabPane.setTitleAt(1, "Update Library");
    this.tabPane.setTitleAt(2, "Remove Library");

    this.tabPane.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent changeEvent) {
        if (tabPane.getSelectedIndex() == 1) {
          new ProgressDialog(projectPath, updateLibraryTable).update();
        }
      }
    });
    try {
      List<String[]> projectLibraryList =
          new ProjectDependencyHandler(projectPath).getProjectLibraryList();

      createAvailableLibraryTree(projectPath);
      createLibraryTable(updateLibraryTable, TableType.Update,
          projectLibraryList);
      createLibraryTable(removeLibraryTable, TableType.Remove,
          projectLibraryList);
    } catch (IOException e) {
      JOptionPane.showMessageDialog(this, "Unable to retrieve library list.",
          "Error", JOptionPane.ERROR_MESSAGE);
    }

    IdeUtils.packAndCenter(this);
  }

  private void createAvailableLibraryTree(final String projectPath) {
    DefaultMutableTreeNode root = new DefaultMutableTreeNode("libraries");
    List<String> libs =
        new ProjectDependencyHandler(projectPath).getAvailableLibraries("/");

    for (String lib : libs) {
      DefaultMutableTreeNode node = new DefaultMutableTreeNode(lib);
      node.add(new DefaultMutableTreeNode(""));
      root.add(node);
    }

    availableLibraryTree.setCellRenderer(new DefaultTreeCellRenderer() {
      public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                    boolean selected,
                                                    boolean expanded,
                                                    boolean leaf, int row,
                                                    boolean hasFocus) {
        return ((DefaultMutableTreeNode) value)
            .getUserObject() instanceof JCheckBox ?
            (JCheckBox) ((DefaultMutableTreeNode) value).getUserObject() :
            super.getTreeCellRendererComponent(tree, value, selected,
                expanded, leaf, row, hasFocus);
      }
    });

    availableLibraryTree.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        int selRow = availableLibraryTree.getRowForLocation(e.getX(), e.getY());
        TreePath selPath =
            availableLibraryTree.getPathForLocation(e.getX(), e.getY());

        if (selPath == null) {
          return;
        }

        Component component = availableLibraryTree
            .getCellRenderer().getTreeCellRendererComponent(
            availableLibraryTree, selPath.getLastPathComponent(), true,
            true, true, selRow, true);

        if (component instanceof JCheckBox) {
          boolean selected = !((JCheckBox) component).isSelected();

          String flattenedPath = getFlattenedPath(selPath).toString();
          flattenedPath =
              flattenedPath.substring(1, flattenedPath.lastIndexOf('/'));

          for (int i = 0; i < 2; ++i) {
            flattenedPath = flattenedPath
                .substring(0, flattenedPath.lastIndexOf('/')) + ":" +
                flattenedPath.substring(flattenedPath.lastIndexOf('/') + 1);
          }

          flattenedPath = flattenedPath.replace('/', '.');

          if (selected) {
            addLibraryList.add(flattenedPath);
          } else {
            addLibraryList.remove(flattenedPath);
          }

          ((JCheckBox) component).setSelected(selected);
        }
      }
    });

    availableLibraryTree.addTreeExpansionListener(new TreeExpansionListener() {
      public void treeExpanded(TreeExpansionEvent event) {
        DefaultMutableTreeNode selectedNode =
            ((DefaultMutableTreeNode) event.getPath().getLastPathComponent());

        if (Character
            .isDigit(selectedNode.getUserObject().toString().charAt(0))) {
          return;
        }

        StringBuffer path = getFlattenedPath(event.getPath());

        List<String> libs =
            new ProjectDependencyHandler(projectPath)
                .getAvailableLibraries(path.toString());
        selectedNode.removeAllChildren();

        for (String lib : libs) {
          DefaultMutableTreeNode newChild;

          if (Character.isDigit(lib.charAt(0))) {
            newChild = new DefaultMutableTreeNode(new JCheckBox(lib));
          } else {
            newChild = new DefaultMutableTreeNode(lib);
            newChild.add(new DefaultMutableTreeNode(""));
          }

          selectedNode.add(newChild);
        }

        ((DefaultTreeModel) availableLibraryTree.getModel())
            .reload(selectedNode);
      }

      public void treeCollapsed(TreeExpansionEvent treeExpansionEvent) {
      }
    });

    ((DefaultTreeModel) availableLibraryTree.getModel()).setRoot(root);
  }

  private StringBuffer getFlattenedPath(TreePath treePath) {
    StringBuffer path = new StringBuffer("/");
    for (Object node : treePath.getPath()) {
      Object userObject = ((DefaultMutableTreeNode) node).getUserObject();
      String text =
          userObject instanceof JCheckBox ? ((JCheckBox) userObject).getText() :
              userObject.toString();
      if (!"libraries".equals(text) && !"".equals(text)) {
        path.append(text).append("/");
      }
    }
    return path;
  }

  private void createLibraryTable(final JTable table, TableType type,
                                  List<String[]> libraryList) {
    DefaultTableModel model = new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return col == 0;
      }

      public Class<?> getColumnClass(int i) {
        return getValueAt(0, i).getClass();
      }
    };

    String[] colNames = {"*", "Group ID", "Artifact ID", "Version"};

    if (TableType.Update.equals(type)) {
      colNames = new String[]{"*", "Group ID", "Artifact ID", "Version",
          "Latest Version"};
    }

    for (String name : colNames) {
      model.addColumn(name);
    }

    for (String[] libInfo : libraryList) {
      Object[] values =
          new Object[]{Boolean.FALSE, libInfo[0], libInfo[1], libInfo[2]};

      if (TableType.Update.equals(type)) {
        values =
            new Object[]{Boolean.FALSE, libInfo[0], libInfo[1], libInfo[2], ""};
      }

      model.addRow(values);
    }

    table.setModel(model);
    table.getColumnModel().getColumn(0).setMaxWidth(25);

    table.getTableHeader().addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent mouseEvent) {
        if (table.getTableHeader().columnAtPoint(mouseEvent.getPoint()) != 0) {
          return;
        }

        boolean check = true;

        for (int row = 0; row < table.getRowCount(); ++row) {
          if (Boolean.TRUE.equals(table.getValueAt(row, 0))) {
            check = false;
            break;
          }
        }

        for (int row = 0; row < table.getRowCount(); ++row) {
          table.setValueAt(check, row, 0);
        }
      }
    });
  }

  private void onOK() {
    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public void getData(ProjectDependenciesAction data) {
    boolean isAdd = this.tabPane.getSelectedIndex() == 0;
    boolean isUpdate = this.tabPane.getSelectedIndex() == 1;
    List<String[]> dependencies = new ArrayList<String[]>(5);

    if (isAdd) {
      for (String lib : addLibraryList) {
        dependencies.add(lib.split(":"));
      }
    } else {
      JTable table = isUpdate ? updateLibraryTable : removeLibraryTable;
      for (int row = 0; row < table.getRowCount(); ++row) {
        if (Boolean.TRUE.equals(table.getValueAt(row, 0))) {
          if (isUpdate) {
            dependencies.add(new String[]{(String) table.getValueAt(row, 1),
                (String) table.getValueAt(row, 2),
                (String) table.getValueAt(row, 3),
                (String) table.getValueAt(row, 4)});
          } else {
            dependencies.add(new String[]{(String) table.getValueAt(row, 1),
                (String) table.getValueAt(row, 2),
                (String) table.getValueAt(row, 3)});
          }
        }
      }
    }

    data.setTab(this.tabPane.getSelectedIndex());
    data.setDependencies(dependencies);
    data.setAddToBuildPath(addToBuildPathCheckBox.isSelected());
    data.setAddToDeploymentUnit(addToDeploymentUnitCheckBox.isSelected());
    data.setRemoveFromBuildPath(removeFromBuildPathCheckBox.isSelected());
    data.setRemoveFromDeploymentUnit(
        removeFromDeploymentUnitCheckBox.isSelected());
  }

  public boolean isOk() {
    return ok;
  }
}

/**
 * @author Juan Hall
 */
class ProgressDialog extends JDialog {
  private JProgressBar progressBar;
  private JTable updateLibaryTable;
  private String projectPath;
  private JTextArea statusTextArea;

  public ProgressDialog(String projectPath, JTable table) {
    this.setTitle("Finding Latest Dependency Versions");
    this.setModal(true);
    this.updateLibaryTable = table;
    this.projectPath = projectPath;
    progressBar = new JProgressBar(0, table.getRowCount());
    progressBar.setBorderPainted(true);
    statusTextArea = new JTextArea(2, 80);
    statusTextArea.setEditable(false);
    this.setLayout(new GridLayout(2, 1, 5, 5));
    this.getContentPane().add(progressBar);
    this.getContentPane().add(statusTextArea);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

    this.setBounds(((int) screenSize.getWidth() - 300) / 2,
        ((int) screenSize.getHeight() - 100) / 2, 300, 100);
  }

  public void update() {
    Thread t = new Thread() {
      public void run() {
        try {
          for (int i = 0; i < updateLibaryTable.getRowCount(); ++i) {
            updateLibaryTable.setValueAt(
                new ProjectDependencyHandler(
                    projectPath).getLatestDependencyVersion(
                    updateLibaryTable.getValueAt(i, 1).toString(),
                    updateLibaryTable.getValueAt(i, 2).toString()), i,
                4);
            progressBar.setValue(i);
            statusTextArea.setText(String.format(
                "Group ID: %s\nArtifact ID: %s",
                updateLibaryTable.getValueAt(i, 1).toString(),
                updateLibaryTable.getValueAt(i, 2).toString()));
          }
        } finally {
          dispose();
        }
      }
    };
    t.start();
    this.setVisible(true);
  }
}