/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.util;

import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.compiler.CompileStatusNotification;
import com.intellij.openapi.compiler.CompilerManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ModifiableRootModel;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.roots.OrderEntry;
import com.intellij.openapi.roots.OrderRootType;
import com.intellij.openapi.roots.libraries.Library;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.util.PathUtil;
import midstream.core.util.WizardUtils;
import org.apache.tools.ant.ProjectHelper;
import org.dom4j.Document;
import org.dom4j.Node;
import org.springframework.util.FileCopyUtils;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public final class IdeUtils {

  private IdeUtils() {
  }

  public static boolean isPluginProject(Project project) {
    final String projectPath = project.getProjectFilePath().substring(0,
        project.getProjectFilePath().indexOf(
            project.getProjectFile().getName()));
    File buildPropertiesFile =
        new File(projectPath + "build.properties");

    if (buildPropertiesFile.exists()) {
      try {
        return new String(FileCopyUtils.copyToByteArray(buildPropertiesFile),
            "ASCII")
            .startsWith("#Generated by MidStream");
      } catch (IOException ex) {
        Logger.getInstance(IdeUtils.class.getName()).error(ex);
        return false;
      }
    }

    return false;
  }

  public static void reloadProject() {
    VirtualFileManager.getInstance().refresh(true);
  }

  public static void rebuildProject(Project project, final Runnable runner) {
    CompilerManager.getInstance(project)
        .make(new CompileStatusNotification() {
          public void finished(boolean b, int i, int i1) {
            runner.run();
          }
        });
  }

  public static ToolWindow createConsoleWindow(Project project,
                                               JComponent component,
                                               String name) {
    ToolWindowManager manager = ToolWindowManager.getInstance(project);
    ToolWindow window = manager.getToolWindow(name);

    if (window == null) {
      window =
          manager.registerToolWindow(name, component, ToolWindowAnchor.BOTTOM);
      window.setIcon(new ImageIcon(IdeUtils.class.getClassLoader().getResource(
          "midstream/core/resources/icons/action.gif")));

    }

    return window;
  }

  public static VirtualFile getVirtualFile(final String filePath) {
    final VirtualFile[] files = new VirtualFile[1];

    ApplicationManager.getApplication().runWriteAction(new Runnable() {
      public void run() {
        files[0] = LocalFileSystem.getInstance()
            .refreshAndFindFileByPath(
                FileUtil.toSystemIndependentName(filePath));
      }
    });

    return files[0];
  }

  public static void openFile(final Project project, final String fileName) {
    ApplicationManager.getApplication().runWriteAction(new Runnable() {
      public void run() {
        VirtualFile file = getVirtualFile(fileName);

        if (file != null) {
          if (file.isValid()) {
            file.refresh(false, false);
          }

          FileEditorManager.getInstance(project).openFile(file, true);
        } else {
          JOptionPane.showMessageDialog(null, fileName,
              "File Does Not Exist", JOptionPane.INFORMATION_MESSAGE);
        }
      }
    });
  }

  public static JFrame suggestParentWindow(final Project project) {
    return (JFrame) WindowManager.getInstance().suggestParentWindow(project);
  }

  public static void runAntTarget(final String projectPath,
                                  final String target) {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");

    org.apache.tools.ant.Project antProject =
        new org.apache.tools.ant.Project();
    antProject.setUserProperty("ant.file", buildFile.getAbsolutePath());
    antProject.init();
    ProjectHelper helper = ProjectHelper.getProjectHelper();
    antProject.addReference("ant.projectHelper", helper);
    helper.parse(antProject, buildFile);
    antProject.executeTarget(target);
  }

  public static void createLibraries(final Project project,
                                     final String projectPath,
                                     final String[] libs) {
    WizardUtils.addLatestDependencies(projectPath, libs);

    runAntTarget(projectPath, "init");

    updateLibraryBuildPath(projectPath, project, libs, true);
  }

  public static void updateLibraryBuildPath(final String projectPath,
                                            final Project project,
                                            final String[] libs,
                                            final boolean isAdd) {
    getVirtualFile(projectPath + File.separatorChar + "lib");

    ApplicationManager.getApplication().runWriteAction(new Runnable() {
      public void run() {
        Module module = ModuleManager.getInstance(project).getModules()[0];
        ModifiableRootModel moduleModel = ModuleRootManager.getInstance(module)
            .getModifiableModel();

        for (String lib : libs) {
          try {
            if (lib.contains(":")) {
              lib = lib.split(":")[1];
            }

            if (isAdd && !hasLibraryEntry(moduleModel,
                lib.endsWith(".jar") ? lib : WizardUtils.getJarNameFromPom(
                    projectPath, lib))) {
              Library library =
                  moduleModel.getModuleLibraryTable().createLibrary();
              Library.ModifiableModel libraryModel =
                  library.getModifiableModel();
              libraryModel.addRoot("jar://" + projectPath + File.separatorChar +
                  "lib" +
                  File.separatorChar +
                  (lib.endsWith(".jar") ? lib : WizardUtils.getJarNameFromPom(
                      projectPath, lib)) + "!/",
                  OrderRootType.CLASSES);
              libraryModel.commit();
            } else if (!isAdd && hasLibraryEntry(moduleModel,
                lib.endsWith(".jar") ? lib : WizardUtils.getJarNameFromPom(
                    projectPath, lib))) {
              Library library = findLibrary(
                  lib.endsWith(".jar") ? lib : WizardUtils.getJarNameFromPom(
                      projectPath, lib),
                  moduleModel.getModuleLibraryTable().getLibraries());
              if (library != null) {
                moduleModel.getModuleLibraryTable().removeLibrary(library);
              }
            }
          } catch (Throwable ex) {
            Logger.getInstance(this.getClass().getName()).error(ex);
            throw new RuntimeException(ex);
          }
        }

        try {
          moduleModel.commit();
        } catch (Exception ex) {
          Logger.getInstance(this.getClass().getName()).error(ex);
          throw new RuntimeException(ex);
        }

        project.save();
      }
    });
  }

  private static Library findLibrary(String name, Library[] libraries) {
    for (Library lib : libraries) {
      for (String url : lib.getUrls(OrderRootType.CLASSES)) {
        if (url.endsWith(name + "!/")) {
          return lib;
        }
      }
    }

    return null;
  }

  private static boolean hasLibraryEntry(ModifiableRootModel moduleModel,
                                         String lib) {
    for (OrderEntry entry : moduleModel.getOrderEntries()) {
      for (String url : entry.getUrls(OrderRootType.CLASSES)) {
        if (url.endsWith(lib + "!/")) {
          return true;
        }
      }
    }
    return false;
  }

  public static boolean isTableEmpty(JTable table, int col) {
    if (table == null) {
      return true;
    }

    boolean empty = true;

    for (int i = 0; i < table.getRowCount(); ++i) {
      if (table.getValueAt(i, col) != null &&
          !"".equals(table.getValueAt(i, col).toString().trim())) {
        empty = false;
        break;
      }
    }

    return empty;
  }

  public static boolean isLibraryOnBuildPath(final String projectPath,
                                             final Project project,
                                             final String lib) {
    getVirtualFile(projectPath + File.separatorChar + "lib");
    final boolean[] checks = {false};

    ApplicationManager.getApplication().runWriteAction(new Runnable() {
      public void run() {
        Module module = ModuleManager.getInstance(project).getModules()[0];
        ModifiableRootModel moduleModel = ModuleRootManager.getInstance(module)
            .getModifiableModel();

        checks[0] = hasLibraryEntry(moduleModel,
            lib.endsWith(".jar") ? lib : WizardUtils.getJarNameFromPom(
                projectPath, lib));
      }
    });

    return checks[0];
  }

  public static String getPluginId() {
    String pluginId = "MidStream";

    try {
      Document doc;
      File jarPath = new File(PathUtil.getJarPathForClass(IdeUtils.class));

      if (jarPath.isDirectory()) {
        doc = WizardUtils.readXmlDocument(
            new File(jarPath.getParentFile(), "META-INF/plugin.xml"), false);
      } else {
        doc = WizardUtils.readXmlDocument(
            IdeUtils.class.getClassLoader().getResourceAsStream(
                "META-INF/plugin.xml"), false);
      }

      if (doc != null) {
        pluginId = doc.selectSingleNode("//name").getText() + "-" +
            doc.selectSingleNode("//version").getText();
      }
    } catch (IOException ex) {
      Logger.getInstance(IdeUtils.class.getName()).error(ex);
    }

    return pluginId;
  }

  public static void packAndCenter(JDialog dialog) {
    dialog.pack();

    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

    if (dialog.getHeight() > screenSize.getHeight() - 50) {
      dialog.setSize(dialog.getWidth(), (int) screenSize.getHeight() - 50);
    }

    dialog.setLocation((int) ((screenSize.getWidth() - dialog.getWidth()) / 2),
        (int) ((screenSize.getHeight() - dialog.getHeight()) / 2));
  }

  public static void setSystemProperties() {
    File settingsFile =
        new File(System.getProperty("user.home") + File.separatorChar +
            ".m2" + File.separatorChar + "settings.xml");

    try {
      Document doc = WizardUtils.readXmlDocument(settingsFile, false);

      if (doc != null) {
        Node node = doc.selectSingleNode("//proxy/host");

        if (node != null) {
          System.setProperty("proxyHost", node.getText());
        }

        node = doc.selectSingleNode("//proxy/port");

        if (node != null) {
          System.setProperty("proxyPort", node.getText());
        }

        node = doc.selectSingleNode("//proxy/username");

        if (node != null) {
          System.setProperty("proxyUser", node.getText());
        }

        node = doc.selectSingleNode("//proxy/password");

        if (node != null) {
          System.setProperty("proxyPassword", node.getText());
        }
      }
    } catch (IOException e) {
      Logger.getInstance(IdeUtils.class.getName()).error(e);
    }
  }
}
